from django.db import models
from djorm_pgarray.fields import ArrayField

from .base import OCDBase, LinkBase, OCDIDField, RelatedBase
from .people_orgs import Organization, Person
from .jurisdiction import LegislativeSession
from .bill import Bill
from .. import common


class VoteEvent(OCDBase):
    id = OCDIDField(ocd_type='vote')
    identifier = models.CharField(max_length=300, blank=True)
    motion_text = models.TextField()
    motion_classification = ArrayField(dbtype="text")      # enum
    start_date = models.CharField(max_length=19)    # YYYY-MM-DD HH:MM:SS
    end_date = models.CharField(max_length=19, blank=True)    # YYYY-MM-DD

    result = models.CharField(max_length=50, choices=common.VOTE_RESULT_CHOICES)
    organization = models.ForeignKey(Organization, related_name='votes')
    legislative_session = models.ForeignKey(LegislativeSession, related_name='votes')
    bill = models.ForeignKey(Bill, related_name='votes', null=True)

    def __str__(self):
        if self.identifier:
            return '{} in {}'.format(self.identifier, self.legislative_session)
        else:
            return '{} on {}'.format(self.motion_text, self.bill)

    class Meta:
        index_together = [
            ['legislative_session', 'identifier', 'bill'],
            ['legislative_session', 'bill']
        ]


class VoteCount(RelatedBase):
    vote = models.ForeignKey(VoteEvent, related_name='counts')
    option = models.CharField(max_length=50, choices=common.VOTE_OPTION_CHOICES)
    value = models.PositiveIntegerField()


class PersonVote(RelatedBase):
    vote = models.ForeignKey(VoteEvent, related_name='votes')
    option = models.CharField(max_length=50, choices=common.VOTE_OPTION_CHOICES)
    voter_name = models.CharField(max_length=300)
    voter = models.ForeignKey(Person, related_name='votes', null=True)
    note = models.TextField(blank=True)


class VoteSource(LinkBase):
    vote_event = models.ForeignKey(VoteEvent, related_name='sources')



from django.db import models
from djorm_pgarray.fields import ArrayField

from ..common import JURISDICTION_CLASSIFICATION_CHOICES, SESSION_CLASSIFICATION_CHOICES
from .base import OCDBase, OCDIDField, RelatedBase
from .division import Division


class Jurisdiction(OCDBase):
    id = OCDIDField(ocd_type='jurisdiction')
    name = models.CharField(max_length=300)
    url = models.URLField(max_length=2000)
    classification = models.CharField(max_length=50, choices=JURISDICTION_CLASSIFICATION_CHOICES,
                                      default='government', db_index=True)
    feature_flags = ArrayField(dbtype="text")
    division = models.ForeignKey(Division, related_name='jurisdictions', db_index=True)

    def __str__(self):
        return self.name


class LegislativeSession(RelatedBase):
    jurisdiction = models.ForeignKey(Jurisdiction, related_name='legislative_sessions')
    identifier = models.CharField(max_length=100)
    name = models.CharField(max_length=300)
    classification = models.CharField(max_length=100, choices=SESSION_CLASSIFICATION_CHOICES,
                                      blank=True)
    start_date = models.CharField(max_length=10)    # YYYY[-MM[-DD]]
    end_date = models.CharField(max_length=10)    # YYYY[-MM[-DD]]

    def __str__(self):
        return '{} {} Session'.format(self.jurisdiction, self.name)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('opencivicdata', '0003_auto_20150507_1554'),
    ]

    operations = [
        migrations.AlterField(
            model_name='billabstract',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billaction',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billactionrelatedentity',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billdocument',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billdocumentlink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billidentifier',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billsource',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billsponsorship',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billtitle',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billversion',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='billversionlink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventagendaitem',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventagendamedia',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventagendamedialink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventdocument',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventdocumentlink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventlink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventlocation',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventmedia',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventmedialink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventparticipant',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventrelatedentity',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='eventsource',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='legislativesession',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='membershipcontactdetail',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='membershiplink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='organizationcontactdetail',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='organizationidentifier',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='organizationlink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='organizationname',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='organizationsource',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='personcontactdetail',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='personidentifier',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='personlink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='personname',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='personsource',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='personvote',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='postcontactdetail',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='postlink',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='relatedbill',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='votecount',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='votesource',
            name='id',
            field=models.UUIDField(serialize=False, primary_key=True, default=uuid.uuid4, editable=False),
        ),
    ]



from django.db import models, transaction
from .base import OCDBase, LinkBase, OCDIDField, RelatedBase, IdentifierBase
from .division import Division
from .jurisdiction import Jurisdiction
from .. import common
from .merge import start_and_end_dates, common_merge, set_up_merge
#from .merge import common_merge, start_and_end_dates

# abstract models


class ContactDetailBase(RelatedBase):
    type = models.CharField(max_length=50, choices=common.CONTACT_TYPE_CHOICES)
    value = models.CharField(max_length=300)
    note = models.CharField(max_length=300, blank=True)
    label = models.CharField(max_length=300, blank=True)

    def transfer_contact_details(persistent, obsolete):
        for contact in obsolete.contact_details.all():
            matches = persistent.contact_details.filter(type=contact.type,
                                                                   value=contact.value)
            for m in matches:
                #get rid of old contact details that match a new one
                m.delete()

            persistent.contact_details.add(contact)

    class Meta:
        abstract = True

    def __str__(self):
        return '{}: {}'.format(self.get_type_display(), self.value)


class OtherNameBase(RelatedBase):
    name = models.CharField(max_length=500, db_index=True)
    note = models.CharField(max_length=500, blank=True)
    start_date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]
    end_date = models.CharField(max_length=10, blank=True)      # YYYY[-MM[-DD]]

    class Meta:
        abstract = True


# the actual models

class Organization(OCDBase):
    id = OCDIDField(ocd_type='organization')
    name = models.CharField(max_length=300)
    image = models.URLField(blank=True, max_length=2000)
    parent = models.ForeignKey('self', related_name='children', null=True)
    jurisdiction = models.ForeignKey(Jurisdiction, related_name='organizations', null=True)
    classification = models.CharField(max_length=100, blank=True,
                                      choices=common.ORGANIZATION_CLASSIFICATION_CHOICES)
    founding_date = models.CharField(max_length=10, blank=True)     # YYYY[-MM[-DD]]
    dissolution_date = models.CharField(max_length=10, blank=True)  # YYYY[-MM[-DD]]

    def __str__(self):
        return self.name

    # Access all "ancestor" organizations
    def get_parents(self):
        org = self
        while True:
            org = org.parent
            # Django accesses parents lazily, so have to check if one actually exists
            if org:
                yield org
            else:
                break

    @transaction.atomic
    def merge(self, obsolete_obj, force=False):
        persistent_obj = self
        new, old = set_up_merge(persistent_obj, obsolete_obj, 'Organization')

        #don't merge if they have different jurisdictions
        if (old.jurisdiction_id != ''
                and new.jurisdiction_id != ''
                and old.jurisdiction_id != new.jurisdiction_id
                and not force):
            raise AssertionError("Refusing to merge orgs from different jurisdictions.")

        valid_jurisdiction = new.jurisdiction or old.jurisdiction
        setattr(persistent_obj, "jurisdiction", valid_jurisdiction)

        
        ContactDetailBase.transfer_contact_details(persistent_obj, obsolete_obj)
        IdentifierBase.transfer_identifiers(persistent_obj, obsolete_obj)
        LinkBase.transfer_links(persistent_obj, obsolete_obj, "links")
        LinkBase.transfer_links(persistent_obj, obsolete_obj, "sources")

        # memberships
        # if a membership with the same post and org exists in old:
        for new_mem in new.memberships.all():
            filter_keys = {'person': new_mem.person,
                           'post': new_mem.post}
            for old_mem in old.memberships.filter(**filter_keys):
                # if they don't overlap, keep them both

                if (new_mem.end_date and old_mem.end_date
                        and new_mem.end_date < old_mem.start_date):
                    # the memberships don't overlap, don't merge
                    pass
                elif (old_mem.end_date and new_mem.start_date
                        and old_mem.end_date < new_mem.start_date):
                    # the memberships don't overlap, don't merge
                    pass
                else:
                    new_mem.organization = persistent_obj
                    old_mem.merge(new_mem)

        #transfer all orgs that have obselete as a parent
        for child in obsolete_obj.children.all():
            child.parent = persistent_obj
            child.save()


        common_merge(persistent_obj, obsolete_obj, new, old, ['name',
                     'image', 'classification', 'extras',
                     'founding_date', 'dissolution_date', 'parent_id'],
                     [], ['jurisdiction_id'], False)

    class Meta:
        index_together = [
            ['jurisdiction', 'classification', 'name'],
            ['classification', 'name'],
        ]


class OrganizationIdentifier(IdentifierBase):
    organization = models.ForeignKey(Organization, related_name='identifiers')

    def __str__(self):
        tmpl = '%s identifies %s'
        return tmpl % (self.identifier, self.organization)


class OrganizationName(OtherNameBase):
    organization = models.ForeignKey(Organization, related_name='other_names')


class OrganizationContactDetail(ContactDetailBase):
    organization = models.ForeignKey(Organization, related_name='contact_details')


class OrganizationLink(LinkBase):
    organization = models.ForeignKey(Organization, related_name='links')


class OrganizationSource(LinkBase):
    organization = models.ForeignKey(Organization, related_name='sources')


class Post(OCDBase):
    id = OCDIDField(ocd_type='post')
    label = models.CharField(max_length=300)
    role = models.CharField(max_length=300, blank=True)
    organization = models.ForeignKey(Organization, related_name='posts')
    division = models.ForeignKey(Division, related_name='posts', null=True, default=None)
    start_date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]
    end_date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]

    class Meta:
        index_together = [
            ['organization', 'label']
        ]

    def __str__(self):
        return '{} - {} - {}'.format(self.role, self.label, self.organization)


class PostContactDetail(ContactDetailBase):
    post = models.ForeignKey(Post, related_name='contact_details')


class PostLink(LinkBase):
    post = models.ForeignKey(Post, related_name='links')


class Person(OCDBase):
    id = OCDIDField(ocd_type='person')
    name = models.CharField(max_length=300, db_index=True)
    sort_name = models.CharField(max_length=100, default='')
    family_name = models.CharField(max_length=100, blank=True)
    given_name = models.CharField(max_length=100, blank=True)

    image = models.URLField(blank=True, max_length=2000)
    gender = models.CharField(max_length=100, blank=True)
    summary = models.CharField(max_length=500, blank=True)
    national_identity = models.CharField(max_length=300, blank=True)
    biography = models.TextField(blank=True)
    birth_date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]
    death_date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]

    def __str__(self):
        return self.name

    def add_other_name(self, name, note=""):
        PersonName.objects.create(name=name,
                        note=note,
                        person_id=self.id)

    class Meta:
        verbose_name_plural = "people"

    @transaction.atomic
    def merge(self, obsolete_obj, force=False):
        persistent_obj = self
        new, old = set_up_merge(persistent_obj, obsolete_obj, 'Person')

        if (old.birth_date != ''
                and new.birth_date != ''
                and old.birth_date != new.birth_date
                and not force):
            raise AssertionError("Refusing to merge people with different birthdays.")

        valid_birthdate = new.birth_date or old.birth_date
        setattr(persistent_obj, "birth_date", valid_birthdate)

        if persistent_obj.name not in persistent_obj.other_names.all():
            persistent_obj.add_other_name(persistent_obj.name)
        if obsolete_obj.name not in persistent_obj.other_names.all():
            persistent_obj.add_other_name(obsolete_obj.name)

        for n in obsolete_obj.other_names.all():
            persistent_obj.other_names.add(n)
        
        ContactDetailBase.transfer_contact_details(persistent_obj, obsolete_obj)
        IdentifierBase.transfer_identifiers(persistent_obj, obsolete_obj)
        LinkBase.transfer_links(persistent_obj, obsolete_obj, "links")
        LinkBase.transfer_links(persistent_obj, obsolete_obj, "sources")

        # memberships
        # if a membership with the same post and org exists in old:
        for new_mem in new.memberships.all():
            filter_keys = {'organization': new_mem.organization,
                           'post': new_mem.post}
            for old_mem in old.memberships.filter(**filter_keys):
                # if they don't overlap, keep them both

                if (new_mem.end_date and old_mem.end_date
                        and new_mem.end_date < old_mem.start_date):
                    # the memberships don't overlap, don't merge
                    pass
                elif (old_mem.end_date and new_mem.start_date
                        and old_mem.end_date < new_mem.start_date):
                    # the memberships don't overlap, don't merge
                    pass
                else:
                    new_mem.person = persistent_obj
                    old_mem.merge(new_mem)


        common_merge(persistent_obj, obsolete_obj, new, old, ['sort_name', 'family_name',
                     'given_name', 'image', 'gender', 'summary',
                     'national_identity', 'biography',
                     'death_date', 'extras'], [],
                    ['birth_date', 'name'], False)


class PersonIdentifier(IdentifierBase):
    person = models.ForeignKey(Person, related_name='identifiers')


class PersonName(OtherNameBase):
    person = models.ForeignKey(Person, related_name='other_names')


class PersonContactDetail(ContactDetailBase):
    person = models.ForeignKey(Person, related_name='contact_details')


class PersonLink(LinkBase):
    person = models.ForeignKey(Person, related_name='links')


class PersonSource(LinkBase):
    person = models.ForeignKey(Person, related_name='sources')


class Membership(OCDBase):
    id = OCDIDField(ocd_type='membership')
    organization = models.ForeignKey(Organization, related_name='memberships')
    person = models.ForeignKey(Person, related_name='memberships')
    post = models.ForeignKey(Post, related_name='memberships', null=True)
    on_behalf_of = models.ForeignKey(Organization, related_name='memberships_on_behalf_of',
                                     null=True)
    label = models.CharField(max_length=300, blank=True)
    role = models.CharField(max_length=300, blank=True)
    start_date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]
    end_date = models.CharField(max_length=10, blank=True)      # YYYY[-MM[-DD]]

    class Meta:
        index_together = [
            ['organization', 'person', 'label', 'post']
        ]

    def __str__(self):
        return '{} in {} ({})'.format(self.person, self.organization, self.role)

    @transaction.atomic
    def merge(self, obsolete_obj, force=False):
        #current obj persists, keeping newer fields when they conflict
        persistent_obj = self
        new, old = set_up_merge(persistent_obj, obsolete_obj, 'Membership')
        if force == False:

            if (new.post and old.post and new.post != old.post):
                msg = "Memberships have different posts. Refusing to merge "\
                      "without forcing. This is probably a bad idea. "\
                      "Consider yourself warned."
                raise AssertionError(msg)

            if (new.person and old.person and new.person != old.person and not force):
                msg = "Memberships have different people. Refusing to merge "\
                      "without forcing. This is probably a bad idea. "\
                      "Consider yourself warned."
                raise AssertionError(msg)

            if (new.organization and old.organization
                and new.organization != old.organization and not force):
                    msg = "Memberships have different organizations. Refusing "\
                          "to merge without forcing. This is probably a bad idea. "\
                          "Consider yourself warned."
                    raise AssertionError(msg)

        # TODO check for human edited fields

        keep_old, keep_new, custom_fields = start_and_end_dates(persistent_obj,
                                                                obsolete_obj,
                                                                force=force,
                                                                keep_old=[],
                                                                keep_new=[],
                                                                custom_fields=[])
        keep_new.extend(['organization_id', 'person_id', 'post_id', 'on_behalf_of_id',
                        'label', 'role'])

        ContactDetailBase.transfer_contact_details(old, new)
        LinkBase.transfer_links(old, new, "links")

        keep_new.append('extras')

        common_merge(persistent_obj, obsolete_obj, new, old,
                     keep_new, keep_old, custom_fields, False)


class MembershipContactDetail(ContactDetailBase):
    membership = models.ForeignKey(Membership, related_name='contact_details')


class MembershipLink(LinkBase):
    membership = models.ForeignKey(Membership, related_name='links')



import sys
import pytest
from django.core.management import call_command
from opencivicdata.models import Division


@pytest.mark.django_db
@pytest.mark.skipif(sys.version_info < (3, 3), reason="requires python3.3")
def test_loaddivisions():
    assert Division.objects.count() == 0
    call_command('loaddivisions', 'in')
    assert Division.objects.count() > 0
    assert Division.objects.count() == Division.objects.filter(country='in').count()
    assert Division.objects.filter(country='in', subtype1='state', subtype2='').count() == 28

    # Include a (very weak) check for idempotency
    call_command('loaddivisions', 'in')
    assert Division.objects.filter(country='in', subtype1='state', subtype2='').count() == 28



from django.db import models
from djorm_pgarray.fields import ArrayField

from .base import (OCDBase, LinkBase, OCDIDField, RelatedBase, RelatedEntityBase, MimetypeLinkBase,
                   IdentifierBase)
from .people_orgs import Organization
from .jurisdiction import LegislativeSession
from .. import common


class Bill(OCDBase):
    id = OCDIDField(ocd_type='bill')
    legislative_session = models.ForeignKey(LegislativeSession, related_name='bills')
    identifier = models.CharField(max_length=100)

    title = models.TextField()

    from_organization = models.ForeignKey(Organization, related_name='bills', null=True)
    classification = ArrayField(dbtype="text")      # check that array values are in enum?
    subject = ArrayField(dbtype="text")

    def __str__(self):
        return '{} in {}'.format(self.identifier, self.legislative_session)

    class Meta:
        index_together = [
            ['from_organization', 'legislative_session', 'identifier'],
        ]


class BillAbstract(RelatedBase):
    bill = models.ForeignKey(Bill, related_name='abstracts')
    abstract = models.TextField()
    note = models.TextField(blank=True)
    date = models.TextField(max_length=10, blank=True)  # YYYY[-MM[-DD]]


class BillTitle(RelatedBase):
    bill = models.ForeignKey(Bill, related_name='other_titles')
    title = models.TextField()
    note = models.TextField(blank=True)


class BillIdentifier(IdentifierBase):
    bill = models.ForeignKey(Bill, related_name='other_identifiers')
    note = models.TextField(blank=True)


class BillAction(RelatedBase):
    bill = models.ForeignKey(Bill, related_name='actions')
    organization = models.ForeignKey(Organization, related_name='actions')
    description = models.TextField()
    date = models.CharField(max_length=10)    # YYYY[-MM[-DD]]
    classification = ArrayField(dbtype="text")      # enum
    order = models.PositiveIntegerField()

    class Meta:
        ordering = ['order']


class BillActionRelatedEntity(RelatedEntityBase):
    action = models.ForeignKey(BillAction, related_name='related_entities')


class RelatedBill(RelatedBase):
    bill = models.ForeignKey(Bill, related_name='related_bills')
    related_bill = models.ForeignKey(Bill, related_name='related_bills_reverse', null=True)
    identifier = models.CharField(max_length=100)
    # not a FK in case we don't know the session yet
    legislative_session = models.CharField(max_length=100)
    relation_type = models.CharField(max_length=100, choices=common.BILL_RELATION_TYPE_CHOICES)

    def __str__(self):
        return 'relationship of {} to {} ({})'.format(self.bill, self.related_bill,
                                                      self.relation_type)


class BillSponsorship(RelatedEntityBase):
    bill = models.ForeignKey(Bill, related_name='sponsorships')
    primary = models.BooleanField(default=False)
    classification = models.CharField(max_length=100)   # enum?

    def __str__(self):
        return '{} ({}) sponsorship of {}'.format(self.name, self.entity_type, self.bill)


class BillDocument(RelatedBase):
    bill = models.ForeignKey(Bill, related_name='documents')
    note = models.CharField(max_length=300)
    date = models.CharField(max_length=10)    # YYYY[-MM[-DD]]


class BillVersion(RelatedBase):
    bill = models.ForeignKey(Bill, related_name='versions')
    note = models.CharField(max_length=300)
    date = models.CharField(max_length=10)    # YYYY[-MM[-DD]]


class BillDocumentLink(MimetypeLinkBase):
    document = models.ForeignKey(BillDocument, related_name='links')


class BillVersionLink(MimetypeLinkBase):
    version = models.ForeignKey(BillVersion, related_name='links')


class BillSource(LinkBase):
    bill = models.ForeignKey(Bill, related_name='sources')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('opencivicdata', '0004_auto_20150610_1600'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bill',
            name='from_organization',
            field=models.ForeignKey(null=True, to='opencivicdata.Organization', related_name='bills'),
        ),
        migrations.AlterField(
            model_name='bill',
            name='legislative_session',
            field=models.ForeignKey(to='opencivicdata.LegislativeSession', related_name='bills'),
        ),
        migrations.AlterField(
            model_name='billabstract',
            name='bill',
            field=models.ForeignKey(to='opencivicdata.Bill', related_name='abstracts'),
        ),
        migrations.AlterField(
            model_name='billaction',
            name='bill',
            field=models.ForeignKey(to='opencivicdata.Bill', related_name='actions'),
        ),
        migrations.AlterField(
            model_name='billaction',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization', related_name='actions'),
        ),
        migrations.AlterField(
            model_name='billactionrelatedentity',
            name='action',
            field=models.ForeignKey(to='opencivicdata.BillAction', related_name='related_entities'),
        ),
        migrations.AlterField(
            model_name='billdocument',
            name='bill',
            field=models.ForeignKey(to='opencivicdata.Bill', related_name='documents'),
        ),
        migrations.AlterField(
            model_name='billdocumentlink',
            name='document',
            field=models.ForeignKey(to='opencivicdata.BillDocument', related_name='links'),
        ),
        migrations.AlterField(
            model_name='billidentifier',
            name='bill',
            field=models.ForeignKey(to='opencivicdata.Bill', related_name='other_identifiers'),
        ),
        migrations.AlterField(
            model_name='billsource',
            name='bill',
            field=models.ForeignKey(to='opencivicdata.Bill', related_name='sources'),
        ),
        migrations.AlterField(
            model_name='billsponsorship',
            name='bill',
            field=models.ForeignKey(to='opencivicdata.Bill', related_name='sponsorships'),
        ),
        migrations.AlterField(
            model_name='billtitle',
            name='bill',
            field=models.ForeignKey(to='opencivicdata.Bill', related_name='other_titles'),
        ),
        migrations.AlterField(
            model_name='billversion',
            name='bill',
            field=models.ForeignKey(to='opencivicdata.Bill', related_name='versions'),
        ),
        migrations.AlterField(
            model_name='billversionlink',
            name='version',
            field=models.ForeignKey(to='opencivicdata.BillVersion', related_name='links'),
        ),
        migrations.AlterField(
            model_name='event',
            name='jurisdiction',
            field=models.ForeignKey(to='opencivicdata.Jurisdiction', related_name='events'),
        ),
        migrations.AlterField(
            model_name='eventagendaitem',
            name='event',
            field=models.ForeignKey(to='opencivicdata.Event', related_name='agenda'),
        ),
        migrations.AlterField(
            model_name='eventagendamedia',
            name='agenda_item',
            field=models.ForeignKey(to='opencivicdata.EventAgendaItem', related_name='media'),
        ),
        migrations.AlterField(
            model_name='eventagendamedialink',
            name='media',
            field=models.ForeignKey(to='opencivicdata.EventAgendaMedia', related_name='links'),
        ),
        migrations.AlterField(
            model_name='eventdocument',
            name='event',
            field=models.ForeignKey(to='opencivicdata.Event', related_name='documents'),
        ),
        migrations.AlterField(
            model_name='eventdocumentlink',
            name='document',
            field=models.ForeignKey(to='opencivicdata.EventDocument', related_name='links'),
        ),
        migrations.AlterField(
            model_name='eventlink',
            name='event',
            field=models.ForeignKey(to='opencivicdata.Event', related_name='links'),
        ),
        migrations.AlterField(
            model_name='eventlocation',
            name='jurisdiction',
            field=models.ForeignKey(to='opencivicdata.Jurisdiction', related_name='event_locations'),
        ),
        migrations.AlterField(
            model_name='eventmedia',
            name='event',
            field=models.ForeignKey(to='opencivicdata.Event', related_name='media'),
        ),
        migrations.AlterField(
            model_name='eventmedialink',
            name='media',
            field=models.ForeignKey(to='opencivicdata.EventMedia', related_name='links'),
        ),
        migrations.AlterField(
            model_name='eventparticipant',
            name='event',
            field=models.ForeignKey(to='opencivicdata.Event', related_name='participants'),
        ),
        migrations.AlterField(
            model_name='eventrelatedentity',
            name='agenda_item',
            field=models.ForeignKey(to='opencivicdata.EventAgendaItem', related_name='related_entities'),
        ),
        migrations.AlterField(
            model_name='eventsource',
            name='event',
            field=models.ForeignKey(to='opencivicdata.Event', related_name='sources'),
        ),
        migrations.AlterField(
            model_name='jurisdiction',
            name='division',
            field=models.ForeignKey(to='opencivicdata.Division', related_name='jurisdictions'),
        ),
        migrations.AlterField(
            model_name='legislativesession',
            name='classification',
            field=models.CharField(max_length=100, choices=[('primary', 'Primary'), ('special', 'Special')], blank=True),
        ),
        migrations.AlterField(
            model_name='legislativesession',
            name='jurisdiction',
            field=models.ForeignKey(to='opencivicdata.Jurisdiction', related_name='legislative_sessions'),
        ),
        migrations.AlterField(
            model_name='membership',
            name='on_behalf_of',
            field=models.ForeignKey(null=True, to='opencivicdata.Organization', related_name='memberships_on_behalf_of'),
        ),
        migrations.AlterField(
            model_name='membership',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization', related_name='memberships'),
        ),
        migrations.AlterField(
            model_name='membership',
            name='person',
            field=models.ForeignKey(to='opencivicdata.Person', related_name='memberships'),
        ),
        migrations.AlterField(
            model_name='membership',
            name='post',
            field=models.ForeignKey(null=True, to='opencivicdata.Post', related_name='memberships'),
        ),
        migrations.AlterField(
            model_name='membershipcontactdetail',
            name='membership',
            field=models.ForeignKey(to='opencivicdata.Membership', related_name='contact_details'),
        ),
        migrations.AlterField(
            model_name='membershiplink',
            name='membership',
            field=models.ForeignKey(to='opencivicdata.Membership', related_name='links'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='jurisdiction',
            field=models.ForeignKey(null=True, to='opencivicdata.Jurisdiction', related_name='organizations'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='parent',
            field=models.ForeignKey(null=True, to='opencivicdata.Organization', related_name='children'),
        ),
        migrations.AlterField(
            model_name='organizationcontactdetail',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization', related_name='contact_details'),
        ),
        migrations.AlterField(
            model_name='organizationidentifier',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization', related_name='identifiers'),
        ),
        migrations.AlterField(
            model_name='organizationlink',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization', related_name='links'),
        ),
        migrations.AlterField(
            model_name='organizationname',
            name='end_date',
            field=models.CharField(max_length=10, blank=True),
        ),
        migrations.AlterField(
            model_name='organizationname',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization', related_name='other_names'),
        ),
        migrations.AlterField(
            model_name='organizationname',
            name='start_date',
            field=models.CharField(max_length=10, blank=True),
        ),
        migrations.AlterField(
            model_name='organizationsource',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization', related_name='sources'),
        ),
        migrations.AlterField(
            model_name='person',
            name='biography',
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name='person',
            name='gender',
            field=models.CharField(max_length=100, blank=True),
        ),
        migrations.AlterField(
            model_name='person',
            name='national_identity',
            field=models.CharField(max_length=300, blank=True),
        ),
        migrations.AlterField(
            model_name='person',
            name='summary',
            field=models.CharField(max_length=500, blank=True),
        ),
        migrations.AlterField(
            model_name='personcontactdetail',
            name='person',
            field=models.ForeignKey(to='opencivicdata.Person', related_name='contact_details'),
        ),
        migrations.AlterField(
            model_name='personidentifier',
            name='person',
            field=models.ForeignKey(to='opencivicdata.Person', related_name='identifiers'),
        ),
        migrations.AlterField(
            model_name='personlink',
            name='person',
            field=models.ForeignKey(to='opencivicdata.Person', related_name='links'),
        ),
        migrations.AlterField(
            model_name='personname',
            name='end_date',
            field=models.CharField(max_length=10, blank=True),
        ),
        migrations.AlterField(
            model_name='personname',
            name='person',
            field=models.ForeignKey(to='opencivicdata.Person', related_name='other_names'),
        ),
        migrations.AlterField(
            model_name='personname',
            name='start_date',
            field=models.CharField(max_length=10, blank=True),
        ),
        migrations.AlterField(
            model_name='personsource',
            name='person',
            field=models.ForeignKey(to='opencivicdata.Person', related_name='sources'),
        ),
        migrations.AlterField(
            model_name='personvote',
            name='vote',
            field=models.ForeignKey(to='opencivicdata.VoteEvent', related_name='votes'),
        ),
        migrations.AlterField(
            model_name='personvote',
            name='voter',
            field=models.ForeignKey(null=True, to='opencivicdata.Person', related_name='votes'),
        ),
        migrations.AlterField(
            model_name='post',
            name='division',
            field=models.ForeignKey(null=True, to='opencivicdata.Division', default=None, related_name='posts'),
        ),
        migrations.AlterField(
            model_name='post',
            name='end_date',
            field=models.CharField(max_length=10, blank=True),
        ),
        migrations.AlterField(
            model_name='post',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization', related_name='posts'),
        ),
        migrations.AlterField(
            model_name='post',
            name='start_date',
            field=models.CharField(max_length=10, blank=True),
        ),
        migrations.AlterField(
            model_name='postcontactdetail',
            name='post',
            field=models.ForeignKey(to='opencivicdata.Post', related_name='contact_details'),
        ),
        migrations.AlterField(
            model_name='postlink',
            name='post',
            field=models.ForeignKey(to='opencivicdata.Post', related_name='links'),
        ),
        migrations.AlterField(
            model_name='relatedbill',
            name='bill',
            field=models.ForeignKey(to='opencivicdata.Bill', related_name='related_bills'),
        ),
        migrations.AlterField(
            model_name='relatedbill',
            name='related_bill',
            field=models.ForeignKey(null=True, to='opencivicdata.Bill', related_name='related_bills_reverse'),
        ),
        migrations.AlterField(
            model_name='votecount',
            name='vote',
            field=models.ForeignKey(to='opencivicdata.VoteEvent', related_name='counts'),
        ),
        migrations.AlterField(
            model_name='voteevent',
            name='bill',
            field=models.ForeignKey(null=True, to='opencivicdata.Bill', related_name='votes'),
        ),
        migrations.AlterField(
            model_name='voteevent',
            name='legislative_session',
            field=models.ForeignKey(to='opencivicdata.LegislativeSession', related_name='votes'),
        ),
        migrations.AlterField(
            model_name='voteevent',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization', related_name='votes'),
        ),
        migrations.AlterField(
            model_name='votesource',
            name='vote_event',
            field=models.ForeignKey(to='opencivicdata.VoteEvent', related_name='sources'),
        ),
    ]



import pytest
from opencivicdata.models import (Jurisdiction, LegislativeSession,                 # noqa
                                  Division, Organization,                           # noqa
                                  OrganizationIdentifier, OrganizationName,         # noqa
                                  OrganizationContactDetail, OrganizationSource,    # noqa
                                  Person, PersonIdentifier, PersonName,             # noqa
                                  PersonContactDetail, PersonLink, PersonSource,    # noqa
                                  Post, PostContactDetail, PostLink, Membership,    # noqa
                                  MembershipContactDetail, MembershipLink)          # noqa
from django.core.exceptions import ValidationError


def test_division_subtypes_from_id():

    # simplest case
    assert Division.subtypes_from_id('ocd-division/country:us') == ({'country': 'us'}, 1)

    # ocd-divison part is optional
    assert Division.subtypes_from_id('country:us/state:ak/county:wild') == (
        {'country': 'us', 'subtype1': 'state', 'subid1': 'ak', 'subtype2': 'county',
         'subid2': 'wild'}, 3)

    # country is not optional
    with pytest.raises(ValueError):
        Division.subtypes_from_id('state:nc/city:raleigh')


@pytest.mark.django_db
def test_division_create():
    division_id = 'ocd-division/country:us/state:ak/county:wild'
    d = Division.objects.create(id=division_id, name='Wild County')
    assert d.country == 'us'
    assert d.subtype1 == 'state'
    assert d.subid1 == 'ak'
    assert d.subtype2 == 'county'
    assert d.subid2 == 'wild'
    assert division_id in str(d)


@pytest.mark.django_db
def test_division_children_of():
    Division.objects.create('ocd-division/country:us', name='US')
    ak = Division.objects.create('ocd-division/country:us/state:ak', name='Alaska')
    Division.objects.create('ocd-division/country:us/state:ak/county:wild', name='Wild')
    Division.objects.create('ocd-division/country:us/state:ak/county:mild', name='Mild')
    Division.objects.create('ocd-division/country:us/state:ak/county:wild/place:a', name='A')
    Division.objects.create('ocd-division/country:us/state:ak/county:wild/place:b', name='B')
    Division.objects.create('ocd-division/country:us/state:ak/county:wild/school:a', name='A')
    Division.objects.create('ocd-division/country:us/state:ak/county:mild/place:a', name='A')
    Division.objects.create('ocd-division/country:us/state:ak/county:mild/place:a/x:y', name='A')

    # simplest ==
    assert Division.objects.children_of('ocd-division/country:us')[0].id == ak.id

    # 3 divisions within wild county
    assert (Division.objects.children_of('ocd-division/country:us/state:ak/county:wild').count() ==
            3)

    # only one school in wild county
    assert Division.objects.children_of('ocd-division/country:us/state:ak/county:wild',
                                        subtype='school').count() == 1

    # 6 divisions beneath alaska up to 2 levels
    assert Division.objects.children_of('ocd-division/country:us/state:ak', depth=2).count() == 6

    # 7 divisions beneath alaska up to 3 levels
    assert Division.objects.children_of('ocd-division/country:us/state:ak', depth=3).count() == 7


@pytest.mark.django_db
def test_ocdid_default():
    o = Organization.objects.create(name='test org')
    assert o.id.startswith('ocd-organization/')
    assert o.pk == o.id
    p = Person.objects.create(name='test person')
    assert p.id.startswith('ocd-person/')


@pytest.mark.django_db
def test_ocdid_default_nondup():
    """ ensure that defaults actually vary """
    p1 = Person(name='test person 1')
    p2 = Person(name='test person 2')
    assert p1.id != p2.id


@pytest.mark.django_db
def test_ocdid_validation_jurisdiction():
    # this fails
    with pytest.raises(ValidationError):
        j = Jurisdiction(name='test juris', id='ocd-division/country:us/test:something/else',
                         url='http://example.com')
        j.full_clean(exclude=['division'])

    # this succeeds
    j = Jurisdiction(name='test juris', id='ocd-jurisdiction/country:us/test:something/else',
                     url='http://example.com')
    j.full_clean(exclude=['division'])


@pytest.mark.django_db
def test_ocdid_validation_other():
    # this test should handle everything that isn't a jurisdiction

    # this succeeds
    o = Organization(name='test org')
    o.full_clean(exclude=['parent', 'jurisdiction'])

    # this raises
    with pytest.raises(ValidationError):
        o = Organization(name='this is a test', id='ocd-organization/3')
        o.full_clean(exclude=['parent', 'jurisdiction'])


@pytest.mark.django_db
def test_organization_get_parents():
    o1 = Organization.objects.create(name='National Organization on Bread-and-Cheese Products')
    o2 = Organization.objects.create(name='Committee on Pizza', parent=o1)
    o3 = Organization.objects.create(name='Subcommittee on Sicilian Pizza', parent=o2)

    assert list(o3.get_parents()) == [o2, o1]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('opencivicdata', '0002_auto_20150131_1021'),
    ]

    operations = [
        migrations.AddField(
            model_name='billabstract',
            name='date',
            field=models.TextField(blank=True, max_length=10),
            preserve_default=True,
        ),
    ]



from django.contrib import admin
from django.template import defaultfilters

from opencivicdata import models
from . import base


@admin.register(models.EventLocation)
class EventLocationAdmin(admin.ModelAdmin):
    pass


class EventLinkInline(base.LinkAdminInline):
    model = models.EventLink


class EventSourceInline(base.LinkAdminInline):
    model = models.EventSource


class EventParticipantInline(base.RelatedEntityInline):
    model = models.EventParticipant
    readonly_fields = ('organization', 'person')


@admin.register(models.Event)
class EventAdmin(admin.ModelAdmin):
    readonly_fields = ('jurisdiction', 'location')
    fields = (
        'name', 'jurisdiction', 'location', 'description',
        'classification', 'status',
        ('start_time', 'end_time'),
        ('timezone', 'all_day'),
        )

    def source_link(self, obj):
        source = obj.sources.filter(url__icontains="meetingdetail").get()
        tmpl = u'<a href="{0}" target="_blank">View source</a>'
        return tmpl.format(source.url)
    source_link.short_description = 'View source'
    source_link.allow_tags = True

    list_display = (
        'jurisdiction', 'name', 'start_time', 'source_link')

    inlines = [
        EventLinkInline,
        EventSourceInline,
        EventParticipantInline,
        ]


@admin.register(models.EventMedia)
class EventMediaAdmin(admin.ModelAdmin):
    pass


@admin.register(models.EventDocument)
class EventDocumentAdmin(admin.ModelAdmin):
    readonly_fields = ('event',)
    list_display = ('event', 'date', 'note')


# @admin.register(models.EventDocumentLink)
# class EventDocumentLinkAdmin(base.MimetypeLinkAdmin):
#    readonly_fields = ('document',)
#    list_display = ('document', 'media_type', 'url',)


@admin.register(models.EventSource)
class EventSourceAdmin(admin.ModelAdmin):
    readonly_fields = ('event',)


@admin.register(models.EventParticipant)
class EventParticipantAdmin(admin.ModelAdmin):
    pass


@admin.register(models.EventAgendaItem)
class EventAgendaItemAdmin(admin.ModelAdmin):
    readonly_fields = ('event',)
    fields = ('event', 'description', 'order', 'subjects', 'notes')

    def get_truncated_description(self, obj):
        return defaultfilters.truncatewords(obj.description, 25)
    get_truncated_description.short_description = 'Description'

    def get_truncated_event_name(self, obj):
        return defaultfilters.truncatewords(obj.event.name, 8)
    get_truncated_event_name.short_description = 'Event Name'

    list_display = ('get_truncated_event_name', 'get_truncated_description')


@admin.register(models.EventRelatedEntity)
class EventRelatedEntityAdmin(admin.ModelAdmin):
    pass


@admin.register(models.EventAgendaMedia)
class EventAgendaMediaAdmin(admin.ModelAdmin):
    pass


@admin.register(models.EventAgendaMediaLink)
class EventAgendaMediaLinkAdmin(admin.ModelAdmin):
    pass



from __future__ import print_function

from django.db import transaction
from django.core.management.base import BaseCommand

from opencivicdata.divisions import Division as FileDivision
from ...models import Division


def to_db(fd):
    """ convert a FileDivision to a Division """
    args, _ = Division.subtypes_from_id(fd.id)
    if fd.sameAs:
        args['redirect_id'] = fd.sameAs
    return Division(id=fd.id, name=fd.name, **args)


def load_divisions(country):
    existing_divisions = Division.objects.filter(country=country)

    country = FileDivision.get('ocd-division/country:{}'.format(country))
    objects = [to_db(country)]

    for child in country.children(levels=100):
        objects.append(to_db(child))

    print('{} divisions found in the CSV, and {} already in the DB'.
          format(len(objects), existing_divisions.count()))

    if set(objects) == set(existing_divisions):
        print('The CSV and the DB contents are exactly the same; no work to be done!')
    else:
        # delete old ids and add new ones all at once
        with transaction.atomic():
            Division.objects.filter(country=country).delete()
            for object_ in objects:
                object_.save()
            # Division.objects.bulk_create(objects, batch_size=10000)
            # XXX: The above line (bulk_create) ends up causing pk consistency
            #      issues when doing an update, even though we did a delete
            #      and the PK is clear. The .save() on each shockingly
            #      works. I'm switching this until this is entirely
            #      understood.
        print('{} divisions created'.format(len(objects)))


class Command(BaseCommand):
    help = 'initialize a pupa database'

    def add_arguments(self, parser):
        parser.add_argument('countries', nargs='+', type=str)

    def handle(self, *args, **options):
        for country in options['countries']:
            load_divisions(country)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import djorm_pgarray.fields


class Migration(migrations.Migration):

    dependencies = [
        ('opencivicdata', '0005_auto_20150622_1455'),
    ]

    operations = [
        migrations.AddField(
            model_name='bill',
            name='locked_fields',
            field=djorm_pgarray.fields.ArrayField(default=[], dbtype='text'),
        ),
        migrations.AddField(
            model_name='event',
            name='locked_fields',
            field=djorm_pgarray.fields.ArrayField(default=[], dbtype='text'),
        ),
        migrations.AddField(
            model_name='jurisdiction',
            name='locked_fields',
            field=djorm_pgarray.fields.ArrayField(default=[], dbtype='text'),
        ),
        migrations.AddField(
            model_name='membership',
            name='locked_fields',
            field=djorm_pgarray.fields.ArrayField(default=[], dbtype='text'),
        ),
        migrations.AddField(
            model_name='organization',
            name='locked_fields',
            field=djorm_pgarray.fields.ArrayField(default=[], dbtype='text'),
        ),
        migrations.AddField(
            model_name='person',
            name='locked_fields',
            field=djorm_pgarray.fields.ArrayField(default=[], dbtype='text'),
        ),
        migrations.AddField(
            model_name='post',
            name='locked_fields',
            field=djorm_pgarray.fields.ArrayField(default=[], dbtype='text'),
        ),
        migrations.AddField(
            model_name='voteevent',
            name='locked_fields',
            field=djorm_pgarray.fields.ArrayField(default=[], dbtype='text'),
        ),
    ]



# flake8: NOQA
from .jurisdiction import Jurisdiction, LegislativeSession
from .division import Division
from .people_orgs import (
    Organization, OrganizationIdentifier, OrganizationName, OrganizationContactDetail,
    OrganizationLink, OrganizationSource,
    Person, PersonIdentifier, PersonName, PersonContactDetail, PersonLink, PersonSource,
    Post, PostContactDetail, PostLink,
    Membership, MembershipContactDetail, MembershipLink
)
from .bill import (Bill, BillAbstract, BillTitle, BillIdentifier, RelatedBill, BillSponsorship,
                   BillDocument, BillVersion, BillDocumentLink, BillVersionLink, BillSource,
                   BillActionRelatedEntity, BillAction)
from .vote import (VoteEvent, VoteCount, PersonVote, VoteSource)
from .event import (Event, EventLocation, EventMedia, EventMediaLink, EventDocument, EventLink,
                    EventSource, EventParticipant, EventAgendaItem, EventRelatedEntity,
                    EventAgendaMedia, EventAgendaMediaLink, EventDocumentLink)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('opencivicdata', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='person',
            name='family_name',
            field=models.CharField(max_length=100, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='person',
            name='given_name',
            field=models.CharField(max_length=100, blank=True),
            preserve_default=True,
        ),
    ]



from waterfall import CascadingUpdate



def common_merge(persistent_obj, obsolete_obj,
          new, old, keep_new, keep_old, custom_fields, force=False):

    #persistent/obsolete and new/old are two sets of pointers
    #to the same object. We need to do this so we can both
    #persist the desired object and transfer newer fields
    #whether the persistent object is the newer or older.
    #ick. but better than alternatives.

    # TODO: check for human_edited fields
    # do stuff that is common to everything
    

    # if we add fields to the model, we catch it here
    new_dict = new.__dict__.copy()
    new_dict.pop('id')
    for f in keep_old:
        if f in new.locked_fields and f not in old.locked_fields:
            field_val = getattr(new, f)
        else:
            field_val = getattr(old, f)
        new_dict.pop(f)
        if field_val:
            setattr(persistent_obj, f, field_val)

    for f in keep_new:
        field_val = new_dict.pop(f)
        if f in old.locked_fields and f not in new.locked_fields:
            field_val = getattr(old, f)
        if field_val:
            setattr(persistent_obj, f, field_val)
    for f in custom_fields:
        new_dict.pop(f)

    for f in obsolete_obj.locked_fields:
        if f not in persistent_obj.locked_fields:
            persistent_obj.locked_fields.append(f)

    new_dict.pop('locked_fields')

    setattr(persistent_obj, "created_at", min(old.created_at, new.created_at))
    new_dict.pop('created_at')
    new_dict.pop('updated_at')
    bad_keys = []
    for k in new_dict:
        if k.startswith("_"):
            bad_keys.append(k)
    for k in bad_keys:
        new_dict.pop(k)

    assert len(new_dict) == 0, \
        "Unexpected fields found: {}".format(', '.join(new_dict.keys()))

    # update foreign keys
    c = CascadingUpdate()
    c.merge_foreign_keys(obsolete_obj, persistent_obj)

    # save old, delete new
    persistent_obj.save()
    obsolete_obj.delete()

    return persistent_obj



def start_and_end_dates(persistent_obj, obsolete_obj, keep_old=[], keep_new=[],
                        custom_fields=[], force=False):
    if not force:
        if ((persistent_obj.end_date and obsolete_obj.start_date
            and persistent_obj.end_date < obsolete_obj.start_date)
            or (obsolete_obj.end_date and persistent_obj.start_date
            and obsolete_obj.end_date < persistent_obj.start_date)):

                msg = "Time ranges do not overlap, will not merge "\
                      "unless forced. Please do not do this unless you "\
                      "are ABSOLUTELY sure you know what you're doing. "\
                      "You've been warned."
                raise AssertionError(msg)

    start_date = (min(persistent_obj.start_date, obsolete_obj.start_date)
                  or persistent_obj.start_date or obsolete_obj.start_date)
    end_date = max(persistent_obj.end_date, obsolete_obj.end_date)
    setattr(persistent_obj, 'start_date', start_date)
    setattr(persistent_obj, 'end_date', end_date)

    return (keep_old, keep_new, ['start_date', 'end_date'])


def set_up_merge(obj1, obj2, obj_type):

    assert (obj1.__class__.__name__ == obj_type and
            obj2.__class__.__name__ == obj_type), \
        "{0} merger needs two {0} objects".format(obj_type)

    # determine which was updated most recently
    if obj1.updated_at > obj2.updated_at:
        return obj1, obj2
    return obj2, obj1



from django.core import urlresolvers
from django.contrib import admin
from django.template import defaultfilters
from .. import models

# Helpers ##########


class ModelAdmin(admin.ModelAdmin):
    """ deletion of top level objects is evil """
    actions = None
    def has_delete_permission(self, request, obj=None):
        return False

    # we probably don't want to add anything through the interface
    def has_add_permission(self, request):
        return False


class ReadOnlyTabularInline(admin.TabularInline):
    def has_add_permission(self, request):
        return False
    can_delete = False


class IdentifierInline(admin.TabularInline):
    fields = readonly_fields = ('identifier', 'scheme')
    extra = 0
    can_delete = False
    verbose_name = "ID from another system"
    verbose_name_plural = "IDs from other systems"
    def has_add_permission(self, request):
        return False


class LinkInline(admin.TabularInline):
    fields = ('url', 'note')
    extra = 0


class ContactDetailInline(admin.TabularInline):
    fields = ('type', 'value', 'note', 'label')
    extra = 0
    verbose_name = "Piece of contact information"
    verbose_name_plural = "Contact information"


class OtherNameInline(admin.TabularInline):
    #fields = ('name', 'note', 'start_date', 'end_date')
    extra = 0
    verbose_name = "Alternate name"
    verbose_name_plural = "Alternate names"

# class MimetypeLinkInline(admin.TabularInline):
#    fields = ('media_type', 'url')
# class RelatedEntityInline(admin.TabularInline):
#    fields = ('name', 'entity_type', 'organization', 'person')

# Divisions & Jurisdictions ##########


@admin.register(models.Division)
class DivisionAdmin(ModelAdmin):
    list_display = ('name', 'id')
    search_fields = list_display
    fields = readonly_fields = ('id', 'name', 'redirect', 'country')
    ordering = ('id',)


class LegislativeSessionInline(ReadOnlyTabularInline):
    model = models.LegislativeSession
    readonly_fields = ('identifier', 'name')
    ordering = ('-identifier',)


@admin.register(models.Jurisdiction)
class JurisdictionAdmin(ModelAdmin):
    list_display = ('name', 'id')
    readonly_fields = fields = ('id', 'name', 'division', 'classification',
                                'feature_flags', 'extras')
    fields = readonly_fields + ('url', )
    ordering = ('id',)
    inlines = [LegislativeSessionInline]

# Organizations and Posts #############


class OrganizationIdentifierInline(IdentifierInline):
    model = models.OrganizationIdentifier


class OrganizationNameInline(OtherNameInline):
    model = models.OrganizationName


class OrganizationContactDetailInline(ContactDetailInline):
    model = models.OrganizationContactDetail


class OrganizationLinkInline(LinkInline):
    model = models.OrganizationLink


class OrganizationSourceInline(LinkInline):
    model = models.OrganizationSource


class PostInline(admin.TabularInline):
    """ a read-only inline for posts here, with links to the real thing """
    model = models.Post
    extra = 0
    fields = readonly_fields = ('label', 'role')
    ordering = ('label',)
    can_delete = False
    show_change_link = True
    
    def has_add_permission(self, request):
        return False


class OrgMembershipInline(ReadOnlyTabularInline):
    model = models.Membership
    fk_name = "organization"
    readonly_fields = ('person', 'post')
    fields = ('person', 'post', 'label', 'role', 'start_date', 'end_date', 'id')
    extra = 0
    can_delete = True


@admin.register(models.Organization)
class OrganizationAdmin(ModelAdmin):
    readonly_fields = ('id', 'classification', 'parent', 'jurisdiction', 'extras')
    fields = (
        'name', 'jurisdiction', 'id', 'classification',
        'parent', ('founding_date', 'dissolution_date'),
        'image', 'extras')
    search_fields = ('name',)

    inlines = [
        OrganizationIdentifierInline,
        OrganizationNameInline,
        OrganizationContactDetailInline,
        OrganizationLinkInline,
        OrganizationSourceInline,
        PostInline,
        OrgMembershipInline,
    ]

    def get_org_name(self, obj):
        parent = obj.parent
        if parent:
            return "{org} ({parent})".format(org=obj.name, parent=parent.name)
        return obj.name
    get_org_name.short_description = "Name"
    get_org_name.allow_tags = True
    get_org_name.admin_order_field = "name"

    def get_jurisdiction(self, obj):
        jurisdiction = obj.jurisdiction
        if jurisdiction:
            admin_url = urlresolvers.reverse('admin:opencivicdata_jurisdiction_change',
                                             args=(jurisdiction.pk,))
            tmpl = '<a href="%s">%s</a>'
            return tmpl % (admin_url, jurisdiction.name)

        return "(none)"

    get_jurisdiction.short_description = 'Jurisdiction'
    get_jurisdiction.allow_tags = True
    get_jurisdiction.admin_order_field = 'jurisdiction__name'

    list_select_related = ('jurisdiction',)
    list_display = ('get_org_name', 'get_jurisdiction', 'classification')
    ordering = ('name',)


class PostContactDetailInline(ContactDetailInline):
    model = models.PostContactDetail


class PostLinkInline(LinkInline):
    model = models.PostLink


@admin.register(models.Post)
class PostAdmin(ModelAdmin):
    readonly_fields = ('id', 'label', 'organization', 'division', 'extras', 'role')
    fields = readonly_fields + (('start_date', 'end_date'), )
    list_display = ('label', 'organization', 'division')
    ordering = ('organization__name',)
    inlines = [
        PostContactDetailInline,
        PostLinkInline,
    ]
    search_fields = ('organization__name', 'label')

# People & Memberships #######


class PersonIdentifierInline(IdentifierInline):
    model = models.PersonIdentifier


class PersonNameInline(OtherNameInline):
    model = models.PersonName


class PersonContactDetailInline(ContactDetailInline):
    model = models.PersonContactDetail


class PersonLinkInline(LinkInline):
    verbose_name = "Related link"
    verbose_name_plural = "Related links"
    model = models.PersonLink


class PersonSourceInline(LinkInline):
    verbose_name = "Source link"
    verbose_name_plural = "Source links"
    model = models.PersonSource


class MembershipInline(ReadOnlyTabularInline):
    model = models.Membership
    readonly_fields = ('organization', 'post')
    fields = ('organization', 'post', 'label', 'role', 'start_date', 'end_date', 'id')
    extra = 0
    can_delete = True


# TODO field locking
@admin.register(models.Person)
class PersonAdmin(ModelAdmin):
    search_fields = ['name']
    readonly_fields = ('id', 'name', 'extras')
    fields = (
        'name', 'id', 'image',
        ('birth_date', 'death_date'),
        ('gender', 'national_identity', 'sort_name', 'summary'),
        'biography', 'extras',
    )
    ordering = ('name',)

    inlines = [
        PersonIdentifierInline,
        PersonNameInline,
        PersonContactDetailInline,
        PersonLinkInline,
        PersonSourceInline,
        MembershipInline
    ]

    def get_memberships(self, obj):
        memberships = obj.memberships.select_related('organization__jurisdiction')
        html = []
        SHOW_N = 5
        for memb in memberships[:SHOW_N]:
            org = memb.organization
            admin_url = urlresolvers.reverse('admin:opencivicdata_organization_change',
                                             args=(org.pk,))
            tmpl = '<a href="%s">%s%s</a>\n'
            html.append(tmpl % (
                admin_url,
                (memb.organization.jurisdiction.name +
                 ": " if memb.organization.jurisdiction else ''),
                memb.organization.name))
        more = len(memberships) - SHOW_N
        if 0 < more:
            html.append('And %d more' % more)
        return '<br/>'.join(html)

    get_memberships.short_description = 'Memberships'
    get_memberships.allow_tags = True

    #list_select_related = ('memberships',)
    list_display = ('name', 'id', 'get_memberships')

# Bills ################


class BillAbstractInline(ReadOnlyTabularInline):
    model = models.BillAbstract
    readonly_fields = ('abstract', 'note')
    can_delete = False


class BillTitleInline(ReadOnlyTabularInline):
    model = models.BillTitle
    readonly_fields = ('title', 'note')
    can_delete = False


class BillIdentifierInline(IdentifierInline):
    model = models.BillIdentifier


class BillActionInline(ReadOnlyTabularInline):
    model = models.BillAction

    def get_related_entities(self, obj):
        ents = obj.related_entities.all()
        # this seems to be a uuid problem, get
        # rid of the below return stmt when uuids are fixed
        return None
        ent_list = [e.name for e in ents]
        return ', '.join(ent_list)

    get_related_entities.short_description = 'Related Entities'
    get_related_entities.allow_tags = True

    list_select_related = ('BillActionRelatedEntity',)
    readonly_fields = ('date', 'organization', 'description', 'get_related_entities')


class RelatedBillInline(ReadOnlyTabularInline):
    model = models.RelatedBill
    fk_name = 'bill'
    readonly_fields = fields = ('identifier', 'legislative_session', 'relation_type')
    extra = 0


class BillSponsorshipInline(ReadOnlyTabularInline):
    model = models.BillSponsorship
    readonly_fields = fields = ('name', 'primary', 'classification')
    extra = 0


class BillDocumentInline(ReadOnlyTabularInline):
    model = models.BillDocument

    def get_link(self, obj):
        return None  # TODO: get rid of this line when uuid issue fixed
        link = obj.links.first()
        return link.url

    get_link.short_description = 'Link'
    get_link.allow_tags = True

    list_select_related = ('BillDocumentLink',)
    readonly_fields = ('note', 'date', 'get_link')


class BillVersionInline(ReadOnlyTabularInline):
    model = models.BillVersion

    def get_link(self, obj):
        return None   # TODO: get rid of this line when uuid issue fixed
        link = obj.links.first()
        return link.url

    get_link.short_description = 'Link'
    get_link.allow_tags = True

    list_select_related = ('BillVersionLink',)
    readonly_fields = ('note', 'date', 'get_link')


class BillSourceInline(ReadOnlyTabularInline):
    readonly_fields = ('url', 'note')
    model = models.BillSource


@admin.register(models.Bill)
class BillAdmin(ModelAdmin):
    readonly_fields = fields = (
        'identifier', 'legislative_session', 'bill_classifications',
        'from_organization', 'title', 'id', 'extras')
    search_fields = ['identifier', 'title']
    list_select_related = (
        #'sources',
        'legislative_session',
        'legislative_session__jurisdiction',
    )
    inlines = [
        BillAbstractInline,
        BillTitleInline,
        BillIdentifierInline,
        BillActionInline,
        BillSponsorshipInline,
        BillSourceInline,
        RelatedBillInline,
        BillVersionInline,
        BillDocumentInline,
    ]

    def bill_classifications(self, obj):
        return ", ".join(obj.classification)

    def get_jurisdiction_name(self, obj):
        return obj.legislative_session.jurisdiction.name
    get_jurisdiction_name.short_description = 'Jurisdiction'

    def get_session_name(self, obj):
        return obj.legislative_session.name
    get_session_name.short_description = 'Session'
    get_session_name.admin_order_field = 'legislative_session__name'

    def source_link(self, obj):
        source = obj.sources.filter(url__icontains="legislationdetail").get()
        tmpl = u'<a href="{0}" target="_blank">View source</a>'
        return tmpl.format(source.url)
    source_link.short_description = 'View source'
    source_link.allow_tags = True

    def get_truncated_sponsors(self, obj):
        spons = ', '.join(s.name for s in obj.sponsorships.all()[:5])
        return defaultfilters.truncatewords(spons, 10)
    get_truncated_sponsors.short_description = 'Sponsors'

    def get_truncated_title(self, obj):
        return defaultfilters.truncatewords(obj.title, 25)
    get_truncated_title.short_description = 'Title'

    list_display = (
        'identifier', 'get_jurisdiction_name',
        'get_session_name', 'get_truncated_sponsors',
        'get_truncated_title', 'source_link')

    list_filter = ('legislative_session__jurisdiction__name',)



# not tests, just Django settings
SECRET_KEY = 'test'
INSTALLED_APPS = ('opencivicdata.apps.BaseConfig',)
DATABASES = {
    'default': {
        'ENGINE': 'django.contrib.gis.db.backends.postgis',
        'NAME': 'test',
        'USER': 'test',
        'PASSWORD': 'test',
        'HOST': 'localhost',
    }
}
MIDDLEWARE_CLASSES = ()



from django.db import models


class DivisionManager(models.Manager):

    def children_of(self, division_id, subtype=None, depth=1):
        query, n = Division.subtypes_from_id(division_id)
        q_objects = []

        # only get children
        if subtype:
            query['subtype{0}'.format(n)] = subtype
        else:
            q_objects.append(~models.Q(**{'subtype{0}'.format(n): ''}))
        q_objects.append(~models.Q(**{'subid{0}'.format(n): ''}))

        # allow for depth wildcards
        n += depth

        # ensure final field is null
        q_objects.append(models.Q(**{'subtype{0}'.format(n): ''}))
        q_objects.append(models.Q(**{'subid{0}'.format(n): ''}))

        return self.filter(*q_objects, **query)

    def create(self, id, name, redirect=None):
        return super(DivisionManager, self).create(id=id, name=name, redirect=redirect,
                                                   **Division.subtypes_from_id(id)[0])


class Division(models.Model):
    objects = DivisionManager()

    id = models.CharField(max_length=300, primary_key=True)
    name = models.CharField(max_length=300)
    redirect = models.ForeignKey('self', null=True)
    country = models.CharField(max_length=2)

    # up to 7 pieces of the id that are searchable
    subtype1 = models.CharField(max_length=50, blank=True)
    subid1 = models.CharField(max_length=100, blank=True)
    subtype2 = models.CharField(max_length=50, blank=True)
    subid2 = models.CharField(max_length=100,  blank=True)
    subtype3 = models.CharField(max_length=50, blank=True)
    subid3 = models.CharField(max_length=100, blank=True)
    subtype4 = models.CharField(max_length=50, blank=True)
    subid4 = models.CharField(max_length=100, blank=True)
    subtype5 = models.CharField(max_length=50, blank=True)
    subid5 = models.CharField(max_length=100, blank=True)
    subtype6 = models.CharField(max_length=50, blank=True)
    subid6 = models.CharField(max_length=100, blank=True)
    subtype7 = models.CharField(max_length=50, blank=True)
    subid7 = models.CharField(max_length=100, blank=True)

    def __str__(self):
        return '{0} ({1})'.format(self.name, self.id)
    __unicode__ = __str__

    @staticmethod
    def subtypes_from_id(division_id):
        pieces = [piece.split(':', 1) for piece in division_id.split('/')]
        fields = {}

        # if it included the ocd-division bit, pop it off
        if pieces[0] == ['ocd-division']:
            pieces.pop(0)

        if pieces[0][0] != 'country':
            raise ValueError('OCD id must start with country')

        fields['country'] = pieces[0][1]

        # add the remaining pieces
        n = 1
        for stype, subid in pieces[1:]:
            fields['subtype{0}'.format(n)] = stype
            fields['subid{0}'.format(n)] = subid
            n += 1

        return fields, n



from django.contrib.gis.db import models
from djorm_pgarray.fields import ArrayField
from .base import OCDBase, LinkBase, OCDIDField, RelatedBase, MimetypeLinkBase, RelatedEntityBase
from .jurisdiction import Jurisdiction
from .bill import Bill
from .vote import VoteEvent


EVENT_STATUS_CHOICES = (
    ('cancelled', 'Cancelled'),
    ('tentative', 'Tentative'),
    ('confirmed', 'Confirmed'),
    ('passed', 'Passed'),
)


class EventMediaBase(RelatedBase):
    note = models.CharField(max_length=300)
    date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]
    offset = models.PositiveIntegerField(null=True)

    class Meta:
        abstract = True


class EventLocation(RelatedBase):
    name = models.CharField(max_length=200)
    url = models.URLField(blank=True, max_length=2000)
    coordinates = models.PointField(null=True)
    jurisdiction = models.ForeignKey(Jurisdiction, related_name='event_locations')

    objects = models.GeoManager()

    def __str__(self):
        return self.name


class Event(OCDBase):
    id = OCDIDField(ocd_type='event')
    name = models.CharField(max_length=300)
    jurisdiction = models.ForeignKey(Jurisdiction, related_name='events')
    description = models.TextField()
    classification = models.CharField(max_length=100)
    start_time = models.DateTimeField()
    timezone = models.CharField(max_length=300)
    end_time = models.DateTimeField(null=True)
    all_day = models.BooleanField(default=False)
    status = models.CharField(max_length=20, choices=EVENT_STATUS_CHOICES)
    location = models.ForeignKey(EventLocation, null=True)

    def __str__(self):
        return self.name

    class Meta:
        index_together = [
            ['jurisdiction', 'start_time', 'name']
        ]


class EventMedia(EventMediaBase):
    event = models.ForeignKey(Event, related_name='media')

    def __str__(self):
        return '%s for %s' % (self.note, self.event)


class EventMediaLink(MimetypeLinkBase):
    media = models.ForeignKey(EventMedia, related_name='links')


class EventDocument(MimetypeLinkBase):
    event = models.ForeignKey(Event, related_name='documents')
    note = models.CharField(max_length=300)
    date = models.CharField(max_length=10)

    def __str__(self):
        tmpl = '{doc.note} for event {doc.event}'
        return tmpl.format(doc=self)


class EventDocumentLink(MimetypeLinkBase):
    document = models.ForeignKey(EventDocument, related_name='links')


class EventLink(LinkBase):
    event = models.ForeignKey(Event, related_name='links')


class EventSource(LinkBase):
    event = models.ForeignKey(Event, related_name='sources')


class EventParticipant(RelatedEntityBase):
    event = models.ForeignKey(Event, related_name='participants')
    note = models.TextField()

    def __str__(self):
        tmpl = '%s at %s'
        return tmpl % (self.name, self.event)


class EventAgendaItem(RelatedBase):
    description = models.TextField()
    order = models.CharField(max_length=100, blank=True)
    subjects = ArrayField(dbtype='text')
    notes = ArrayField(dbtype='text')
    event = models.ForeignKey(Event, related_name='agenda')


class EventRelatedEntity(RelatedEntityBase):
    agenda_item = models.ForeignKey(EventAgendaItem, related_name='related_entities')
    bill = models.ForeignKey(Bill, null=True)
    vote = models.ForeignKey(VoteEvent, null=True)
    note = models.TextField()

    @property
    def entity_name(self):
        if self.entity_type == 'vote' and self.vote_id:
            return self.vote.name
        elif self.entity_type == 'bill' and self.bill_id:
            return self.bill.name
        else:
            return super(EventRelatedEntity, self).entity_name

    @property
    def entity_id(self):
        if self.entity_type == 'vote':
            return self.vote_id
        if self.entity_type == 'bill':
            return self.bill_id
        return super(EventRelatedEntity, self).entity_id


class EventAgendaMedia(EventMediaBase):
    agenda_item = models.ForeignKey(EventAgendaItem, related_name='media')


class EventAgendaMediaLink(MimetypeLinkBase):
    media = models.ForeignKey(EventAgendaMedia, related_name='links')



from django.contrib import admin
from opencivicdata.models import vote as models


class VoteCountInline(admin.TabularInline):
    model = models.VoteCount
    fields = readonly_fields = ('option', 'value')
    extra = 0


class PersonVoteInline(admin.TabularInline):
    model = models.PersonVote
    fields = readonly_fields = (
        'voter', 'voter_name', 'option')
    extra = 0


class VoteSourceInline(admin.TabularInline):
    model = models.VoteSource
    fields = readonly_fields = ('url', 'note')
    extra = 0


@admin.register(models.VoteEvent)
class VoteEventAdmin(admin.ModelAdmin):
    readonly_fields = (
        'bill', 'organization', 'legislative_session', 'id')
    fields = (
        'organization', 'legislative_session', 'bill',
        'result', 'id', 'identifier', 'motion_text',
        'motion_classification', 'start_date', 'end_date',
        'extras')

    list_selected_related = (
        'sources',
        'legislative_session',
        'legislative_session__jurisdiction')

    inlines = [
        VoteCountInline, PersonVoteInline,
        VoteSourceInline]


@admin.register(models.VoteCount)
class VoteCountAdmin(admin.ModelAdmin):
    pass


@admin.register(models.PersonVote)
class PersonVoteAdmin(admin.ModelAdmin):
    list_display = ('voter_name', 'vote')
    readonly_fields = ('vote',)


@admin.register(models.VoteSource)
class VoteSourceAdmin(admin.ModelAdmin):
    pass



from django.test import TestCase
from opencivicdata.models import merge
from opencivicdata.models import (Jurisdiction, LegislativeSession,                 # noqa
                                  Division, Organization,                           # noqa
                                  OrganizationIdentifier, OrganizationName,         # noqa
                                  OrganizationContactDetail, OrganizationSource,    # noqa
                                  Person, PersonIdentifier, PersonName,             # noqa
                                  PersonContactDetail, PersonLink, PersonSource,    # noqa
                                  Post, PostContactDetail, PostLink, Membership,    # noqa
                                  MembershipContactDetail, MembershipLink)          # noqa

class MergeTestCase(TestCase):
    def setUp(self):
        self.person1 = Person.objects.create(name='Barack Obama',
                                            image='example.com',
                                            sort_name='Obama')
        self.organization1 = Organization.objects.create(name="House")
    def test_merge_of_different_classes(self):
        with self.assertRaises(AssertionError):
            merge.set_up_merge(self.person1, self.organization1, 'Person')
        self.assertEquals(Person.objects.count(), 1)
        self.assertEquals(Organization.objects.count(), 1)

class MembershipTestCase(TestCase):
    def setUp(self):
        self.person1 = Person.objects.create(name='Barack Obama',
                                            image='example.com',
                                            sort_name='Obama')
        self.person2 = Person.objects.create(name='Michelle Obama',
                                            image='example.com',
                                            sort_name='Obama')
        self.pid1 = self.person1.id
        self.pid2 = self.person2.id

        self.organization1 = Organization.objects.create(name="House")
        self.organization2 = Organization.objects.create(name="Senate")

        self.oid1 = self.organization1.id
        self.oid2 = self.organization2.id

    def test_membership_people_org_merge(self):
        membership1 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1)
        membership2 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1)
        membership1.merge(membership2)
        self.assertEqual(Membership.objects.count(), 1,
                         "Merge memberships with same person & org.")

    def test_membership_wrong_people(self):
        membership1 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1)
        membership2 = Membership.objects.create(person_id=self.pid2,
                                                organization_id=self.oid1)
        with self.assertRaises(AssertionError):
            membership1.merge(membership2)
        self.assertEqual(Membership.objects.count(), 2,
                         "Memberships w different people should not merge")
        membership1.merge(membership2, force=True)
        self.assertEqual(Membership.objects.count(), 1,
                         "Memberships w different people should merge if forced")
        self.assertEqual(Membership.objects.first().person_id, self.pid2,
                         "Force-merged memberships should retain person from newer membership")

    def test_membership_wrong_org(self):
        membership1 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1)
        membership2 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid2)
        with self.assertRaises(AssertionError):
            membership1.merge(membership2)
        self.assertEqual(Membership.objects.count(), 2,
                         "Memberships w different orgs should not merge")
        membership1.merge(membership2, force=True)
        self.assertEqual(Membership.objects.count(), 1,
                         "Memberships w different orgs should merge if forced")
        self.assertEqual(Membership.objects.first().organization_id, self.oid2,
                         "Force-merged memberships should retain org from newer membership")

    def test_memberships_subset_dates(self):
        membership1 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1,
                                                start_date='2014-01-01',
                                                end_date='2015-01-01')
        membership2 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1,
                                                start_date='2014-02-01',
                                                end_date='2014-03-01')
        membership1.merge(membership2)
        self.assertEqual(Membership.objects.count(), 1,
                         "Memberships w subsetted starts and ends should merge")
        self.assertEqual(Membership.objects.first().start_date, '2014-01-01',
                        "Resulting membership start date should be earliest")
        self.assertEqual(Membership.objects.first().end_date, '2015-01-01',
                        "Resulting membership end date should be latest")

    def test_memberships_overlapping_dates(self):
        membership1 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1,
                                                start_date='2014-01-01',
                                                end_date='2015-01-01')
        membership2 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1,
                                                start_date='2014-02-01',
                                                end_date='2015-03-01')
        membership1.merge(membership2)
        self.assertEqual(Membership.objects.count(), 1,
                         "Memberships w subsetted starts and ends should merge")
        self.assertEqual(Membership.objects.first().start_date, '2014-01-01',
                        "Resulting membership start date should be earliest")
        self.assertEqual(Membership.objects.first().end_date, '2015-03-01',
                        "Resulting membership end date should be latest")

    def test_memberships_nonoverlapping_dates(self):
        membership1 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1,
                                                start_date='2014-01-01',
                                                end_date='2014-02-01')
        membership2 = Membership.objects.create(person_id=self.pid1,
                                                organization_id=self.oid1,
                                                start_date='2014-03-01',
                                                end_date='2014-04-01')
        with self.assertRaises(AssertionError):
            membership1.merge(membership2)
        self.assertEqual(Membership.objects.count(), 2,
                         "Memberships w non-overlapping dates should not merge")
        membership1.merge(membership2, force=True)
        self.assertEqual(Membership.objects.count(), 1,
                         "Memberships w different non-overlapping dates should merge if forced")
        self.assertEqual(Membership.objects.first().start_date, '2014-01-01',
                         "Force-merged memberships should get earliest start date")
        self.assertEqual(Membership.objects.first().end_date, '2014-04-01',
                         "Force-merged memberships should get latest end date")

    def test_memberships_different_posts(self):
        post1 = Post.objects.create(label="post1", organization_id=self.oid1)
        post2 = Post.objects.create(label="post2", organization_id=self.oid1)
        mem1 = Membership.objects.create(post_id=post1.id,
                                         person_id=self.pid1,
                                         organization_id=self.oid1)
        mem2 = Membership.objects.create(post_id=post2.id,
                                         person_id=self.pid1,
                                         organization_id=self.oid1)
        with self.assertRaises(AssertionError):
            mem1.merge(mem2)

class PersonTestCase(TestCase):
    def setUp(self):
        self.person1 = Person.objects.create(name='Barack Obama',
                                        image='example.com',
                                        sort_name='Barack',
                                        gender='M')

        self.person2 = Person.objects.create(name='Barack Obama',
                                        image='example.com',
                                        sort_name='Obama')

        id1 = self.person1.id
        id2 = self.person2.id

        PersonContactDetail.objects.create(type='fax',
                                           value='555-123-4567',
                                           person_id=id1)
        PersonContactDetail.objects.create(type='fax',
                                           value='555-123-4567',
                                           note="Throw out your fax!",
                                           person_id=id2)
        PersonContactDetail.objects.create(type='email',
                                           value='prez@america.gov',
                                           person_id=id2)
        PersonContactDetail.objects.create(type='email',
                                           value='obama@aol.com',
                                           person_id=id1)

        PersonIdentifier.objects.create(scheme='openstates',
                                        identifier='1234',
                                        person_id=id1)
        PersonIdentifier.objects.create(scheme='openstates',
                                        identifier='1234',
                                        person_id=id2)
        PersonIdentifier.objects.create(scheme='openstates',
                                        identifier='4567',
                                        person_id=id1)
        PersonIdentifier.objects.create(scheme='another_source',
                                        identifier='4567',
                                        person_id=id2)

        PersonLink.objects.create(url='whitehouse.gov',
                                  note='official',
                                  person_id=id2)
        PersonLink.objects.create(url='whitehouse.gov',
                                  note='whitehouse',
                                  person_id=id1)


    def test_simple_merge(self):
        self.person1.merge(self.person2)
        obama_set = Person.objects.filter(name='Barack Obama')
        obama = obama_set.first()
        self.assertEqual(len(obama_set), 1,
                         "There should only be one person after merge.")
        self.assertEqual(obama.sort_name, 'Obama',
                         "Sort name should match most recently updated.")
        self.assertEqual(obama.gender, 'M',
                         "Data should trump empty, no matter the update order.")


    def test_merge_with_contact_details(self):
        self.person1.merge(self.person2)
        obama = Person.objects.filter(name='Barack Obama').first()
        self.assertEqual(len(obama.contact_details.all()),3,
                         "New person retains all non-duplicate contact details")
        self.assertEqual(len(obama.contact_details.filter(type="fax")), 1,
                         "Contact details with same value and type merged")
        self.assertEqual(len(obama.contact_details.filter(type="email")), 2,
                         "Contact details of same type not merged if val differs")
        self.assertEqual(obama.contact_details.filter(type="fax").first().note,
                         "Throw out your fax!",
                         "Newer person's contact detail retained when conflict")

    def test_merge_with_identifiers(self):
        self.person1.merge(self.person2)
        obama = Person.objects.filter(name='Barack Obama').first()
        self.assertEqual(len(obama.identifiers.all()), 3,
                         "New person object retains all non-duplicate identifiers")
        self.assertEqual(len(obama.identifiers.filter(identifier='4567')), 2,
                         "Identifiers w same ID but different schema not overwritten")
        self.assertEqual(len(obama.identifiers.filter(identifier='1234')), 1,
                         "If identifiers have same identifier&scheme, only one is kept")

    def test_merge_with_links(self):
        self.person1.merge(self.person2)
        obama = Person.objects.filter(name='Barack Obama').first()
        self.assertEqual(len(obama.links.all()), 1,
                         "New person object retains all non-duplicate identifiers")
        self.assertEqual(obama.links.first().note, 'official',
                         "Link attached to newer person retained when conflict")

    def test_merge_with_memberships(self):
        org1 = Organization.objects.create(name="Org1")
        org2 = Organization.objects.create(name="Org2")
        org3 = Organization.objects.create(name="Org3")
        org4 = Organization.objects.create(name="Org4")
        org5 = Organization.objects.create(name="Org5")

        #these two should merge:

        Membership.objects.create(person_id=self.person1.id,
                                  organization_id=org1.id)
        Membership.objects.create(person_id=self.person2.id,
                                  organization_id=org1.id)

        #these two should merge, and have start/end dates
        Membership.objects.create(person_id=self.person1.id,
                                  organization_id=org2.id)
        Membership.objects.create(person_id=self.person2.id,
                                  organization_id=org2.id,
                                  start_date='2015-01-02',
                                  end_date='2016-01-02')

        #these two should merge
        Membership.objects.create(person_id=self.person1.id,
                                  organization_id=org3.id,
                                  start_date='2015-02-03',
                                  end_date='2015-03-03')
        Membership.objects.create(person_id=self.person2.id,
                                  organization_id=org3.id,
                                  start_date='2015-01-03',
                                  end_date='2016-01-03')

        #these two should merge
        Membership.objects.create(person_id=self.person1.id,
                                  organization_id=org4.id,
                                  start_date='2015-01-04',
                                  end_date='2015-03-04')
        Membership.objects.create(person_id=self.person2.id,
                                  organization_id=org4.id,
                                  start_date='2015-02-15',
                                  end_date='2016-03-04')
        #these two should not merge
        Membership.objects.create(person_id=self.person1.id,
                                  organization_id=org5.id,
                                  start_date='2015-02-05',
                                  end_date='2015-03-05')
        Membership.objects.create(person_id=self.person2.id,
                                  organization_id=org5.id,
                                  start_date='2015-04-15',
                                  end_date='2016-03-05')

        self.person1.merge(self.person2)
        
        obama = Person.objects.filter(name='Barack Obama').first()
        self.assertEqual(obama.memberships.count(), 6,
                         "right number of merges need to happen.")

        org2_mem = obama.memberships.filter(organization_id=org2.id).first()
        self.assertEqual(org2_mem.start_date,'2015-01-02',
                         "start date for org2 memberships is earliest")
        self.assertEqual(org2_mem.end_date,'2016-01-02',
                         "end date for org2 memberships is latest")

        org3_mem = obama.memberships.filter(organization_id=org3.id).first()
        self.assertEqual(org3_mem.start_date,'2015-01-03',
                         "start date for org3 memberships is earliest")
        self.assertEqual(org3_mem.end_date,'2016-01-03',
                         "end date for org3 memberships is latest")

        org4_mem = obama.memberships.filter(organization_id=org4.id).first()
        self.assertEqual(org4_mem.start_date,'2015-01-04',
                         "start date for org4 memberships is latest")
        self.assertEqual(org4_mem.end_date,'2016-03-04',
                         "end date for org4 memberships is latest")

        org5_mems = obama.memberships.filter(organization_id=org5.id)
        self.assertEqual(len(org5_mems), 2,
                         "org5 memberships don't merge")


class OrgTestCase(TestCase):
    def setUp(self):
        self.org1 = Organization.objects.create(name="Org1",
                                                classification='legislature',
                                                founding_date='1776-07-04')
        self.org2 = Organization.objects.create(name="Org2",
                                                classification='government',
                                                dissolution_date='2012-12-21')
        d1 = Division.objects.create(id='ocd-division/country:us',
                                     name='America Online')
        j1 = Jurisdiction.objects.create(name="j1",
                                         url='example.com',
                                         division=d1,
                                         classification='committee',
                                         id='ocd-jurisdiction/country:us/committee')
        j2 = Jurisdiction.objects.create(name="j2",
                                         url='example.com',
                                         division=d1,
                                         id='ocd-jurisdiction/country:us/government')
        self.org3 = Organization.objects.create(name="Org3")
        self.org4 = Organization.objects.create(name="Org4",
                                                jurisdiction_id = j1.id)
        self.org5 = Organization.objects.create(name="Org5",
                                                jurisdiction_id = j1.id)
        self.org6 = Organization.objects.create(name="Org6",
                                                jurisdiction_id = j2.id)

    def test_merge_orgs_simple(self):
        org_id = self.org1.id
        self.org1.merge(self.org2)
        orgs = Organization.objects.all()
        self.assertEqual(len(orgs), 5, "Orgs should merge")
        new_org = Organization.objects.filter(name='Org2').first()
        self.assertEqual(new_org.id, org_id,
                         'Keep persistent org\'s id')
        self.assertEqual(new_org.classification, 'government',
                         'Keep newer org\'s classification')
        self.assertEqual(new_org.founding_date, '1776-07-04',
                         'Data trumps no-data')
        self.assertEqual(new_org.dissolution_date, '2012-12-21',
                         'Data trumps no-data')

    def test_merge_orgs_with_parents(self):
        self.org1.parent_id = self.org3.id
        self.org2.parent_id = self.org4.id
        self.org5.parent_id = self.org3.id
        self.org6.parent_id = self.org4.id
        self.org1.save()
        self.org2.save()
        self.org5.save()
        self.org6.save()
        self.org1.merge(self.org2)
        merged_org = Organization.objects.filter(name='Org2').first()
        two_org_parent = Organization.objects.filter(name='Org4').first()
        one_org_parent = Organization.objects.filter(name='Org3').first()
        self.assertEqual(Organization.objects.count(), 5,
                         'two orgs merged')
        self.assertEqual(merged_org.parent_id, self.org4.id,
                         'merged org gets newer org\'s parent')
        self.assertEqual(two_org_parent.children.count(), 2,
                         'merging doesn\'t impact other parent-child rlns')
        self.assertEqual(one_org_parent.children.count(), 1,
                         'merging removes obsolete org from parent')

    def test_org_merge_with_jurisdictions(self):
        with self.assertRaises(AssertionError):
            self.org4.merge(self.org6)
        self.assertEqual(Organization.objects.count(), 6,
                         'Orgs with different jurisdictions don\'t merge')
        self.org4.merge(self.org5)
        self.assertEqual(Organization.objects.count(), 5,
                         'Orgs w same jurisdiction merge')

class OrgWithParentsTestCase(TestCase):
    def setUp(self):
        self.org1 = Organization.objects.create(name="Org1")
        self.org2 = Organization.objects.create(name="Org2")
        self.org3 = Organization.objects.create(name="Org3",
                                                parent=self.org1)
        self.org4 = Organization.objects.create(name="Org4",
                                                parent=self.org2)

    def test_org_merge_with_parents(self):
        self.org1.merge(self.org2)
        org3 = Organization.objects.filter(name="Org3").first()
        org4 = Organization.objects.filter(name="Org4").first()
        self.assertEqual(org3.parent.name, "Org2")
        self.assertEqual(org4.parent.name, "Org2")
        self.assertEqual(Organization.objects.count(), 3)

class OrgMembershipTestCase(TestCase):
    def setUp(self):
        self.org1 = Organization.objects.create(name="Org1",
                                                classification='legislature',
                                                founding_date='1776-07-04')
        self.org2 = Organization.objects.create(name="Org2",
                                                classification='government',
                                                dissolution_date='2012-12-21')
        person1 = Person.objects.create(name="a")
        person2 = Person.objects.create(name="b")
        Membership.objects.create(person_id=person1.id, organization_id=self.org1.id)
        Membership.objects.create(person_id=person2.id, organization_id=self.org2.id)
        Membership.objects.create(person_id=person1.id, organization_id=self.org2.id)

    def test_merge_orgs_simple(self):
        self.org1.merge(self.org2)
        orgs = Organization.objects.all()
        self.assertEqual(len(orgs), 1, "Orgs should merge")
        new_org = orgs[0]
        self.assertEqual(len(new_org.memberships.all()), 2, "Memberships merge")        

class PersonBirthdayTestCase(TestCase):
  def setUp(self):
    self.p1 = Person.objects.create(name="George", birth_date='17760704')
    self.p2 = Person.objects.create(name="Thomas", birth_date='17760705')

  def test_birthday_mismatch(self):
      with self.assertRaises(AssertionError):
          self.p1.merge(self.p2)


class LockedFieldsTestCase(TestCase):
    def setUp(self):
        self.org1 = Organization.objects.create(name="Org1",
                                                image="org1.jpg",
                                                locked_fields=['image'])
        self.org2 = Organization.objects.create(name="Org2",
                                                image="org2.jpg",
                                                locked_fields=['founding_date'])  
    def test_org_merge_with_locked_fields(self):
        self.org1.merge(self.org2)
        self.assertEqual(Organization.objects.count(), 1,
                         'Orgs locked fields merge')
        merged_org = Organization.objects.first()
        self.assertEqual(merged_org.name, 'Org2',
                         'Keeps newer org\'s name')
        self.assertEqual(merged_org.image, 'org1.jpg',
                         'Keeps locked fields where relevant')
        self.assertEqual(merged_org.locked_fields, ['image', 'founding_date'],
                         'Keeps locked fields where relevant')


import re
import uuid
from django.db import models
from django.core.validators import RegexValidator
from jsonfield import JSONField
from uuidfield import UUIDField
from djorm_pgarray.fields import ArrayField

from .. import common


class OCDIDField(models.CharField):

    def __init__(self, *args, **kwargs):
        self.ocd_type = kwargs.pop('ocd_type')
        if self.ocd_type != 'jurisdiction':
            kwargs['default'] = lambda: 'ocd-{}/{}'.format(self.ocd_type, uuid.uuid4())
            # len('ocd-') + len(ocd_type) + len('/') + len(uuid)
            #       = 4 + len(ocd_type) + 1 + 36
            #       = len(ocd_type) + 41
            kwargs['max_length'] = 41 + len(self.ocd_type)
            regex = '^ocd-' + self.ocd_type + '/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$'
        else:
            kwargs['max_length'] = 300
            regex = common.JURISDICTION_ID_REGEX

        kwargs['primary_key'] = True
        # get pattern property if it exists, otherwise just return the object (hopefully a string)
        msg = 'ID must match ' + getattr(regex, 'pattern', regex)
        kwargs['validators'] = [RegexValidator(regex=regex, message=msg, flags=re.U)]
        super(OCDIDField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(OCDIDField, self).deconstruct()
        if self.ocd_type != 'jurisdiction':
            kwargs.pop('default')
        kwargs.pop('max_length')
        kwargs.pop('primary_key')
        kwargs['ocd_type'] = self.ocd_type
        return (name, path, args, kwargs)


class OCDBase(models.Model):
    """ common base fields across all top-level models """
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    extras = JSONField(default='{}', blank=True)
    locked_fields = ArrayField(dbtype="text", default=[])

    class Meta:
        abstract = True


class RelatedBase(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    class Meta:
        abstract = True


class LinkBase(RelatedBase):
    note = models.CharField(max_length=300, blank=True)
    url = models.URLField(max_length=2000)

    def transfer_links(persistent, obsolete, link_name="links"):
        for l in getattr(obsolete, link_name).all():
            matches = getattr(persistent, link_name).filter(url=l.url)
            for m in matches:
                m.delete()
            getattr(persistent, link_name).add(l)

    class Meta:
        abstract = True


class MimetypeLinkBase(RelatedBase):
    media_type = models.CharField(max_length=100)
    url = models.URLField(max_length=2000)

    class Meta:
        abstract = True


class IdentifierBase(RelatedBase):
    identifier = models.CharField(max_length=300)
    scheme = models.CharField(max_length=300)

    def transfer_identifiers(persistent, obsolete):
        #pass 2 objects with related identifiers, this will combine them
        for i in obsolete.identifiers.all():
            persistent_matches = persistent.identifiers.filter(identifier=i.identifier,
                                                               scheme=i.scheme)

            if len(persistent_matches) == 0:
                persistent.identifiers.add(i)
            else:
                i.delete()



    class Meta:
        abstract = True


class RelatedEntityBase(RelatedBase):
    name = models.CharField(max_length=2000)
    entity_type = models.CharField(max_length=20, blank=True)

    # optionally tied to an organization or person if it was linkable
    organization = models.ForeignKey('Organization', null=True)
    person = models.ForeignKey('Person', null=True)

    @property
    def entity_name(self):
        if self.entity_type == 'organization' and self.organization_id:
            return self.organization.name
        elif self.entity_type == 'person' and self.person_id:
            return self.person.name
        else:
            return self.name

    @property
    def entity_id(self):
        if self.entity_type == 'organization':
            return self.organization_id
        if self.entity_type == 'person':
            return self.person_id
        return None

    class Meta:
        abstract = True



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(name="opencivicdata-django",
      version='0.6.4',
      author="James Turk",
      author_email='jturk@sunlightfoundation.com',
      license="BSD",
      description="python opencivicdata library",
      long_description="",
      url="",
      py_modules=['opencivicdata.apps', 'opencivicdata.common'],
      packages=['opencivicdata.admin',
                'opencivicdata.management',
                'opencivicdata.management.commands',
                'opencivicdata.migrations',
                'opencivicdata.models',
                'opencivicdata.tests'],
      include_package_data=True,
      platforms=["any"],
      classifiers=["Development Status :: 4 - Beta",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: BSD License",
                   "Natural Language :: English",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python :: 3.4",
                   ],
      )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import uuidfield.fields
import django.core.validators
import jsonfield.fields
import django.contrib.gis.db.models.fields
import djorm_pgarray.fields
import opencivicdata.models.base


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Bill',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('extras', jsonfield.fields.JSONField(blank=True, default='{}')),
                ('id', opencivicdata.models.base.OCDIDField(serialize=False, validators=[django.core.validators.RegexValidator(regex='^ocd-bill/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', message='ID must match ^ocd-bill/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', flags=32)], ocd_type='bill')),
                ('identifier', models.CharField(max_length=100)),
                ('title', models.TextField()),
                ('classification', djorm_pgarray.fields.ArrayField(dbtype='text')),
                ('subject', djorm_pgarray.fields.ArrayField(dbtype='text')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillAbstract',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('abstract', models.TextField()),
                ('note', models.TextField(blank=True)),
                ('bill', models.ForeignKey(to='opencivicdata.Bill')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillAction',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('description', models.TextField()),
                ('date', models.CharField(max_length=10)),
                ('classification', djorm_pgarray.fields.ArrayField(dbtype='text')),
                ('order', models.PositiveIntegerField()),
                ('bill', models.ForeignKey(to='opencivicdata.Bill')),
            ],
            options={
                'ordering': ['order'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillActionRelatedEntity',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('name', models.CharField(max_length=2000)),
                ('entity_type', models.CharField(blank=True, max_length=20)),
                ('action', models.ForeignKey(to='opencivicdata.BillAction')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillDocument',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(max_length=300)),
                ('date', models.CharField(max_length=10)),
                ('bill', models.ForeignKey(to='opencivicdata.Bill')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillDocumentLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('media_type', models.CharField(max_length=100)),
                ('url', models.URLField(max_length=2000)),
                ('document', models.ForeignKey(to='opencivicdata.BillDocument')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillIdentifier',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('identifier', models.CharField(max_length=300)),
                ('scheme', models.CharField(max_length=300)),
                ('note', models.TextField(blank=True)),
                ('bill', models.ForeignKey(to='opencivicdata.Bill')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillSource',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('bill', models.ForeignKey(to='opencivicdata.Bill')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillSponsorship',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('name', models.CharField(max_length=2000)),
                ('entity_type', models.CharField(blank=True, max_length=20)),
                ('primary', models.BooleanField(default=False)),
                ('classification', models.CharField(max_length=100)),
                ('bill', models.ForeignKey(to='opencivicdata.Bill')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillTitle',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('title', models.TextField()),
                ('note', models.TextField(blank=True)),
                ('bill', models.ForeignKey(to='opencivicdata.Bill')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillVersion',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(max_length=300)),
                ('date', models.CharField(max_length=10)),
                ('bill', models.ForeignKey(to='opencivicdata.Bill')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BillVersionLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('media_type', models.CharField(max_length=100)),
                ('url', models.URLField(max_length=2000)),
                ('version', models.ForeignKey(to='opencivicdata.BillVersion')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Division',
            fields=[
                ('id', models.CharField(serialize=False, max_length=300, primary_key=True)),
                ('name', models.CharField(max_length=300)),
                ('country', models.CharField(max_length=2)),
                ('subtype1', models.CharField(blank=True, max_length=50)),
                ('subid1', models.CharField(blank=True, max_length=100)),
                ('subtype2', models.CharField(blank=True, max_length=50)),
                ('subid2', models.CharField(blank=True, max_length=100)),
                ('subtype3', models.CharField(blank=True, max_length=50)),
                ('subid3', models.CharField(blank=True, max_length=100)),
                ('subtype4', models.CharField(blank=True, max_length=50)),
                ('subid4', models.CharField(blank=True, max_length=100)),
                ('subtype5', models.CharField(blank=True, max_length=50)),
                ('subid5', models.CharField(blank=True, max_length=100)),
                ('subtype6', models.CharField(blank=True, max_length=50)),
                ('subid6', models.CharField(blank=True, max_length=100)),
                ('subtype7', models.CharField(blank=True, max_length=50)),
                ('subid7', models.CharField(blank=True, max_length=100)),
                ('redirect', models.ForeignKey(null=True, to='opencivicdata.Division')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('extras', jsonfield.fields.JSONField(blank=True, default='{}')),
                ('id', opencivicdata.models.base.OCDIDField(serialize=False, validators=[django.core.validators.RegexValidator(regex='^ocd-event/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', message='ID must match ^ocd-event/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', flags=32)], ocd_type='event')),
                ('name', models.CharField(max_length=300)),
                ('description', models.TextField()),
                ('classification', models.CharField(max_length=100)),
                ('start_time', models.DateTimeField()),
                ('timezone', models.CharField(max_length=300)),
                ('end_time', models.DateTimeField(null=True)),
                ('all_day', models.BooleanField(default=False)),
                ('status', models.CharField(max_length=20, choices=[('cancelled', 'Cancelled'), ('tentative', 'Tentative'), ('confirmed', 'Confirmed'), ('passed', 'Passed')])),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventAgendaItem',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('description', models.TextField()),
                ('order', models.CharField(blank=True, max_length=100)),
                ('subjects', djorm_pgarray.fields.ArrayField(dbtype='text')),
                ('notes', djorm_pgarray.fields.ArrayField(dbtype='text')),
                ('event', models.ForeignKey(to='opencivicdata.Event')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventAgendaMedia',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(max_length=300)),
                ('date', models.CharField(blank=True, max_length=10)),
                ('offset', models.PositiveIntegerField(null=True)),
                ('agenda_item', models.ForeignKey(to='opencivicdata.EventAgendaItem')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventAgendaMediaLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('media_type', models.CharField(max_length=100)),
                ('url', models.URLField(max_length=2000)),
                ('media', models.ForeignKey(to='opencivicdata.EventAgendaMedia')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventDocument',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('media_type', models.CharField(max_length=100)),
                ('url', models.URLField(max_length=2000)),
                ('note', models.CharField(max_length=300)),
                ('date', models.CharField(max_length=10)),
                ('event', models.ForeignKey(to='opencivicdata.Event')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventDocumentLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('media_type', models.CharField(max_length=100)),
                ('url', models.URLField(max_length=2000)),
                ('document', models.ForeignKey(to='opencivicdata.EventDocument')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('event', models.ForeignKey(to='opencivicdata.Event')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventLocation',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('name', models.CharField(max_length=200)),
                ('url', models.URLField(blank=True, max_length=2000)),
                ('coordinates', django.contrib.gis.db.models.fields.PointField(null=True, srid=4326)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='event',
            name='location',
            field=models.ForeignKey(null=True, to='opencivicdata.EventLocation'),
            preserve_default=True,
        ),
        migrations.CreateModel(
            name='EventMedia',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(max_length=300)),
                ('date', models.CharField(blank=True, max_length=10)),
                ('offset', models.PositiveIntegerField(null=True)),
                ('event', models.ForeignKey(to='opencivicdata.Event')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventMediaLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('media_type', models.CharField(max_length=100)),
                ('url', models.URLField(max_length=2000)),
                ('media', models.ForeignKey(to='opencivicdata.EventMedia')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventParticipant',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('name', models.CharField(max_length=2000)),
                ('entity_type', models.CharField(blank=True, max_length=20)),
                ('note', models.TextField()),
                ('event', models.ForeignKey(to='opencivicdata.Event')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventRelatedEntity',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('name', models.CharField(max_length=2000)),
                ('entity_type', models.CharField(blank=True, max_length=20)),
                ('note', models.TextField()),
                ('agenda_item', models.ForeignKey(to='opencivicdata.EventAgendaItem')),
                ('bill', models.ForeignKey(null=True, to='opencivicdata.Bill')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EventSource',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('event', models.ForeignKey(to='opencivicdata.Event')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Jurisdiction',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('extras', jsonfield.fields.JSONField(blank=True, default='{}')),
                ('id', opencivicdata.models.base.OCDIDField(serialize=False, validators=[django.core.validators.RegexValidator(regex='^ocd-jurisdiction/country:[a-z]{2}(/[^\\W\\d]+:[\\w.~-]+)*/\\w+$', message='ID must match ^ocd-jurisdiction/country:[a-z]{2}(/[^\\W\\d]+:[\\w.~-]+)*/\\w+$', flags=32)], ocd_type='jurisdiction')),
                ('name', models.CharField(max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('classification', models.CharField(max_length=50, db_index=True, choices=[('government', 'Government'), ('legislature', 'Legislature'), ('executive', 'Executive'), ('school_system', 'School System')], default='government')),
                ('feature_flags', djorm_pgarray.fields.ArrayField(dbtype='text')),
                ('division', models.ForeignKey(to='opencivicdata.Division')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='eventlocation',
            name='jurisdiction',
            field=models.ForeignKey(to='opencivicdata.Jurisdiction'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='event',
            name='jurisdiction',
            field=models.ForeignKey(to='opencivicdata.Jurisdiction'),
            preserve_default=True,
        ),
        migrations.AlterIndexTogether(
            name='event',
            index_together=set([('jurisdiction', 'start_time', 'name')]),
        ),
        migrations.CreateModel(
            name='LegislativeSession',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('identifier', models.CharField(max_length=100)),
                ('name', models.CharField(max_length=300)),
                ('classification', models.CharField(max_length=100, choices=[('primary', 'Primary'), ('special', 'Special')])),
                ('start_date', models.CharField(max_length=10)),
                ('end_date', models.CharField(max_length=10)),
                ('jurisdiction', models.ForeignKey(to='opencivicdata.Jurisdiction')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='bill',
            name='legislative_session',
            field=models.ForeignKey(to='opencivicdata.LegislativeSession'),
            preserve_default=True,
        ),
        migrations.CreateModel(
            name='Membership',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('extras', jsonfield.fields.JSONField(blank=True, default='{}')),
                ('id', opencivicdata.models.base.OCDIDField(serialize=False, validators=[django.core.validators.RegexValidator(regex='^ocd-membership/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', message='ID must match ^ocd-membership/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', flags=32)], ocd_type='membership')),
                ('label', models.CharField(blank=True, max_length=300)),
                ('role', models.CharField(blank=True, max_length=300)),
                ('start_date', models.CharField(blank=True, max_length=10)),
                ('end_date', models.CharField(blank=True, max_length=10)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MembershipContactDetail',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('type', models.CharField(max_length=50, choices=[('address', 'Postal Address'), ('email', 'Email'), ('url', 'URL'), ('fax', 'Fax'), ('text', 'Text Phone'), ('voice', 'Voice Phone'), ('video', 'Video Phone'), ('pager', 'Pager'), ('textphone', 'Device for people with hearing impairment')])),
                ('value', models.CharField(max_length=300)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('label', models.CharField(blank=True, max_length=300)),
                ('membership', models.ForeignKey(to='opencivicdata.Membership')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MembershipLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('membership', models.ForeignKey(to='opencivicdata.Membership')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('extras', jsonfield.fields.JSONField(blank=True, default='{}')),
                ('id', opencivicdata.models.base.OCDIDField(serialize=False, validators=[django.core.validators.RegexValidator(regex='^ocd-organization/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', message='ID must match ^ocd-organization/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', flags=32)], ocd_type='organization')),
                ('name', models.CharField(max_length=300)),
                ('image', models.URLField(blank=True, max_length=2000)),
                ('classification', models.CharField(blank=True, max_length=100, choices=[('legislature', 'Legislature'), ('executive', 'Executive'), ('upper', 'Upper Chamber'), ('lower', 'Lower Chamber'), ('party', 'Party'), ('committee', 'Committee'), ('commission', 'Commission')])),
                ('founding_date', models.CharField(blank=True, max_length=10)),
                ('dissolution_date', models.CharField(blank=True, max_length=10)),
                ('jurisdiction', models.ForeignKey(null=True, to='opencivicdata.Jurisdiction')),
                ('parent', models.ForeignKey(null=True, to='opencivicdata.Organization')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='membership',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='membership',
            name='on_behalf_of',
            field=models.ForeignKey(null=True, to='opencivicdata.Organization'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='eventrelatedentity',
            name='organization',
            field=models.ForeignKey(null=True, to='opencivicdata.Organization'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='eventparticipant',
            name='organization',
            field=models.ForeignKey(null=True, to='opencivicdata.Organization'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='billsponsorship',
            name='organization',
            field=models.ForeignKey(null=True, to='opencivicdata.Organization'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='billactionrelatedentity',
            name='organization',
            field=models.ForeignKey(null=True, to='opencivicdata.Organization'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='billaction',
            name='organization',
            field=models.ForeignKey(to='opencivicdata.Organization'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='bill',
            name='from_organization',
            field=models.ForeignKey(null=True, to='opencivicdata.Organization'),
            preserve_default=True,
        ),
        migrations.AlterIndexTogether(
            name='bill',
            index_together=set([('from_organization', 'legislative_session', 'identifier')]),
        ),
        migrations.AlterIndexTogether(
            name='organization',
            index_together=set([('jurisdiction', 'classification', 'name'), ('classification', 'name')]),
        ),
        migrations.CreateModel(
            name='OrganizationContactDetail',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('type', models.CharField(max_length=50, choices=[('address', 'Postal Address'), ('email', 'Email'), ('url', 'URL'), ('fax', 'Fax'), ('text', 'Text Phone'), ('voice', 'Voice Phone'), ('video', 'Video Phone'), ('pager', 'Pager'), ('textphone', 'Device for people with hearing impairment')])),
                ('value', models.CharField(max_length=300)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('label', models.CharField(blank=True, max_length=300)),
                ('organization', models.ForeignKey(to='opencivicdata.Organization')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='OrganizationIdentifier',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('identifier', models.CharField(max_length=300)),
                ('scheme', models.CharField(max_length=300)),
                ('organization', models.ForeignKey(to='opencivicdata.Organization')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='OrganizationLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('organization', models.ForeignKey(to='opencivicdata.Organization')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='OrganizationName',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('name', models.CharField(max_length=500, db_index=True)),
                ('note', models.CharField(blank=True, max_length=500)),
                ('start_date', models.CharField(max_length=10)),
                ('end_date', models.CharField(max_length=10)),
                ('organization', models.ForeignKey(to='opencivicdata.Organization')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='OrganizationSource',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('organization', models.ForeignKey(to='opencivicdata.Organization')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Person',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('extras', jsonfield.fields.JSONField(blank=True, default='{}')),
                ('id', opencivicdata.models.base.OCDIDField(serialize=False, validators=[django.core.validators.RegexValidator(regex='^ocd-person/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', message='ID must match ^ocd-person/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', flags=32)], ocd_type='person')),
                ('name', models.CharField(max_length=300, db_index=True)),
                ('sort_name', models.CharField(max_length=100, default='')),
                ('image', models.URLField(blank=True, max_length=2000)),
                ('gender', models.CharField(max_length=100)),
                ('summary', models.CharField(max_length=500)),
                ('national_identity', models.CharField(max_length=300)),
                ('biography', models.TextField()),
                ('birth_date', models.CharField(blank=True, max_length=10)),
                ('death_date', models.CharField(blank=True, max_length=10)),
            ],
            options={
                'verbose_name_plural': 'people',
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='membership',
            name='person',
            field=models.ForeignKey(to='opencivicdata.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='eventrelatedentity',
            name='person',
            field=models.ForeignKey(null=True, to='opencivicdata.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='eventparticipant',
            name='person',
            field=models.ForeignKey(null=True, to='opencivicdata.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='billsponsorship',
            name='person',
            field=models.ForeignKey(null=True, to='opencivicdata.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='billactionrelatedentity',
            name='person',
            field=models.ForeignKey(null=True, to='opencivicdata.Person'),
            preserve_default=True,
        ),
        migrations.CreateModel(
            name='PersonContactDetail',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('type', models.CharField(max_length=50, choices=[('address', 'Postal Address'), ('email', 'Email'), ('url', 'URL'), ('fax', 'Fax'), ('text', 'Text Phone'), ('voice', 'Voice Phone'), ('video', 'Video Phone'), ('pager', 'Pager'), ('textphone', 'Device for people with hearing impairment')])),
                ('value', models.CharField(max_length=300)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('label', models.CharField(blank=True, max_length=300)),
                ('person', models.ForeignKey(to='opencivicdata.Person')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PersonIdentifier',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('identifier', models.CharField(max_length=300)),
                ('scheme', models.CharField(max_length=300)),
                ('person', models.ForeignKey(to='opencivicdata.Person')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PersonLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('person', models.ForeignKey(to='opencivicdata.Person')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PersonName',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('name', models.CharField(max_length=500, db_index=True)),
                ('note', models.CharField(blank=True, max_length=500)),
                ('start_date', models.CharField(max_length=10)),
                ('end_date', models.CharField(max_length=10)),
                ('person', models.ForeignKey(to='opencivicdata.Person')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PersonSource',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('person', models.ForeignKey(to='opencivicdata.Person')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PersonVote',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('option', models.CharField(max_length=50, choices=[('yes', 'Yes'), ('no', 'No'), ('absent', 'Absent'), ('abstain', 'Abstain'), ('not voting', 'Not Voting'), ('paired', 'Paired'), ('excused', 'Excused'), ('other', 'Other')])),
                ('voter_name', models.CharField(max_length=300)),
                ('note', models.TextField(blank=True)),
                ('voter', models.ForeignKey(null=True, to='opencivicdata.Person')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('extras', jsonfield.fields.JSONField(blank=True, default='{}')),
                ('id', opencivicdata.models.base.OCDIDField(serialize=False, validators=[django.core.validators.RegexValidator(regex='^ocd-post/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', message='ID must match ^ocd-post/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', flags=32)], ocd_type='post')),
                ('label', models.CharField(max_length=300)),
                ('role', models.CharField(blank=True, max_length=300)),
                ('start_date', models.CharField(max_length=10)),
                ('end_date', models.CharField(max_length=10)),
                ('division', models.ForeignKey(null=True, to='opencivicdata.Division', default=None)),
                ('organization', models.ForeignKey(to='opencivicdata.Organization')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='membership',
            name='post',
            field=models.ForeignKey(null=True, to='opencivicdata.Post'),
            preserve_default=True,
        ),
        migrations.AlterIndexTogether(
            name='membership',
            index_together=set([('organization', 'person', 'label', 'post')]),
        ),
        migrations.AlterIndexTogether(
            name='post',
            index_together=set([('organization', 'label')]),
        ),
        migrations.CreateModel(
            name='PostContactDetail',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('type', models.CharField(max_length=50, choices=[('address', 'Postal Address'), ('email', 'Email'), ('url', 'URL'), ('fax', 'Fax'), ('text', 'Text Phone'), ('voice', 'Voice Phone'), ('video', 'Video Phone'), ('pager', 'Pager'), ('textphone', 'Device for people with hearing impairment')])),
                ('value', models.CharField(max_length=300)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('label', models.CharField(blank=True, max_length=300)),
                ('post', models.ForeignKey(to='opencivicdata.Post')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PostLink',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('post', models.ForeignKey(to='opencivicdata.Post')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='RelatedBill',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('identifier', models.CharField(max_length=100)),
                ('legislative_session', models.CharField(max_length=100)),
                ('relation_type', models.CharField(max_length=100, choices=[('companion', 'Companion'), ('prior-session', 'Prior Session'), ('replaced-by', 'Replaced By'), ('replaces', 'Replaces')])),
                ('bill', models.ForeignKey(to='opencivicdata.Bill')),
                ('related_bill', models.ForeignKey(null=True, to='opencivicdata.Bill')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='VoteCount',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('option', models.CharField(max_length=50, choices=[('yes', 'Yes'), ('no', 'No'), ('absent', 'Absent'), ('abstain', 'Abstain'), ('not voting', 'Not Voting'), ('paired', 'Paired'), ('excused', 'Excused'), ('other', 'Other')])),
                ('value', models.PositiveIntegerField()),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='VoteEvent',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('extras', jsonfield.fields.JSONField(blank=True, default='{}')),
                ('id', opencivicdata.models.base.OCDIDField(serialize=False, validators=[django.core.validators.RegexValidator(regex='^ocd-vote/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', message='ID must match ^ocd-vote/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', flags=32)], ocd_type='vote')),
                ('identifier', models.CharField(blank=True, max_length=300)),
                ('motion_text', models.TextField()),
                ('motion_classification', djorm_pgarray.fields.ArrayField(dbtype='text')),
                ('start_date', models.CharField(max_length=19)),
                ('end_date', models.CharField(blank=True, max_length=19)),
                ('result', models.CharField(max_length=50, choices=[('pass', 'Pass'), ('fail', 'Fail')])),
                ('bill', models.ForeignKey(null=True, to='opencivicdata.Bill')),
                ('legislative_session', models.ForeignKey(to='opencivicdata.LegislativeSession')),
                ('organization', models.ForeignKey(to='opencivicdata.Organization')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='votecount',
            name='vote',
            field=models.ForeignKey(to='opencivicdata.VoteEvent'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='personvote',
            name='vote',
            field=models.ForeignKey(to='opencivicdata.VoteEvent'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='eventrelatedentity',
            name='vote',
            field=models.ForeignKey(null=True, to='opencivicdata.VoteEvent'),
            preserve_default=True,
        ),
        migrations.AlterIndexTogether(
            name='voteevent',
            index_together=set([('legislative_session', 'identifier', 'bill'), ('legislative_session', 'bill')]),
        ),
        migrations.CreateModel(
            name='VoteSource',
            fields=[
                ('id', uuidfield.fields.UUIDField(unique=True, max_length=32, editable=False, primary_key=True, serialize=False, blank=True)),
                ('note', models.CharField(blank=True, max_length=300)),
                ('url', models.URLField(max_length=2000)),
                ('vote_event', models.ForeignKey(to='opencivicdata.VoteEvent')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
    ]
